﻿/* Inst-FMbiblio.ahk ------ UTF-8 -----------------------------------------------------------------
                Install FMbiblio into the user environment
Attention     For each project this must be adapted individually
Comment       Normally compiled using the compiler by running !_Make-distribution
Tasks         - Application scripts will be copied into the FM-user-area and optionally the Adobe script folder
              - A shortcut will point to the program after installation
Reference     Traduction française avec l'aide de https://www.deepl.com/translator par Klaus Daube
AHK version   1.1.26.00
              Assure that AutoHotkeySC.bin is the same as  Unicode 32-bit.bin
              Only this way the compiled scripts run on both x64 and x32 systems
Copyleft      2018 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
History       2018-01-17 creation
              2021-05-11 Start converstion to DDD coding standard, localisation by XML files
              2022-01-29 harmonisation with other suites
*/ ; // ===========================================================================================

#Requires AutoHotkey v1.1
#Singleinstance Force
If !A_IsAdmin ; must be first in script (see H:\Utilities\DDDutils\Test-ahks\Run-as-admin.ahk)
	{           ; see *verb in AHK help --- here the context-menu Run as Administrator
  try Run *RunAs "%A_ScriptFullPath%"
  catch
    sysLanguage := SubStr(A_Language, 3)
    If (sysLanguage == "07") {          ; 07: German
      MsgBox, Das skript wurde abgebrochen, als es nach erhöhter berechtigung fragte.`nBitte das script mit administrator rechten starten.
    } Else If (sysLanguage == "0C") {   ; 0C: French
      MsgBox, Le script a été annulé lorsqu'il a demandé d'élever les droits d'accès`nÉcutez à nouveau le script avec des droits admin.
    } Else {                            ; 09: English
      MsgBox, The script has been Cancelled when it asked to elevate access rights`nPlease Run the script  again with admin rights.
    }
  ExitApp
	} ; ---------------------------------------------------------------------------------------------

L0_SetUp:
SetWorkingDir %A_ScriptDir%                                 ; files are in subdirs
Global   appName                                            ; FMxxxx
      , appDescr
      , appIcon                                             ; FM-puzzle.ico
      , appTitle
;     , appHint
      , appVersion                                          ; 1.2
      , auxDir
      , bOption1 = false, bOption2 = false
      , btnCancel, btnInstall, btnUnInst, btnHelp
      , cfgFile     := "config.ini"
;     , exampleDir  := A_ScriptDir . "\FMpppp-examples"    ; <<<<<<<<<<<<< to be adapted <<<<<<<<<
      , FMlang                                              ; de, en, fr
      , FMpgm                                               ; H:\ ... FrameMaker 2017\FrameMaker.exe
      , FMhome                                              ; no terminating \
      , FMfullversion                                       ; 14.0.3.500
      , FMappDir    := A_AppData .   "\Adobe\FrameMaker\"
      , FMiniDir
      , FMscriptDir
      , FMautoRun
      , FMtarget
      , helpFile
      , instHint
      , installDir                                          ; C:\users\…\D+DD\FMpppp
      , option1, option2
      , srcDir      := A_ScriptDir . "\InstallThis"
      , scriptDir   := A_ScriptDir
      , shortCutDir := A_StartMenu . "\D+DD"                ; must be created
      , userIni

L1_Initialise:
  IniRead, FMtarget,   %cfgFile%, Project,  FMtarget
  FMhome      := FunGetFMprops(FMtarget)                    ; set FMlang, FMpgm, FMfullversion
  FMiniDir    := FMappDir . FMtarget
  userIni     := FMiniDir . "\maker.ini"
  FMautoRun   := FMiniDir . "\StartUp"
  GetCfgData()                                              ; read from E:\_DDDprojects\FMxxxx\InstallFMxxxx\config.ini
  FMscriptDir := A_MyDocuments . "\Adobe Scripts\" . appName
  installDir  := A_AppData .   "\D+DD\" . appName
  iconPath    := appIcon
  FileGetTime, appDate, %A_ScriptFullPath%, M
  FormatTime,  appDate, %appDate%, yyyy-MM-dd HH:mm
  appInf   := "Version " . appVersion   . " / " . appDate   ; file date of this script/exe

L2_:

L3_UserInterface:
Menu, Tray, Icon, %iconPath%
Gui, Font, s10, Verdana
Gui, Add, Text,     x200              , %appInf%
Gui, Add, Text,     x005 w470         , %appDescr%
Gui, Add, Text,     w470              , %instHint%%FMtarget%
Gui, Add, Checkbox, w470 vbOption1    , %option1%
Gui, Add, Checkbox, w470 vbOption2    , %option2%

Gui, Add, Button,   x005 w100  gMenuHelp,          %btnHelp%
Gui, Add, Button,   x+20 w100  gMenuInstallation,  %btnInstall%
Gui, Add, Button,   x+20 w100  gMenuUninstall,     %btnUnInst%
Gui, Add, Button,   x+20 w100  gCancel,            %btnCancel%

Gui, Show,          h215 w480,                              %appTitle%
Return

GuiClose:
MenuCancel:
ExitApp

MenuHelp:
	Run, http://www.daube.ch/docu/%helpFile%
Return

MenuInstallation:
  Gui, Submit
  If (bOption1 == 0 && bOption2 == 0) {
   Gui, Destroy
   GoTO L3_UserInterface
  }
; MsgBox, bOption1 = %bOption1%   bOption2 = %bOption2%
; PrepareFiles()
  InstallFiles()
  ModifyMakerIni()
; ModifyUserData()                      ; specific in FMnotes
  CreateShortCut()
  Finalize()
ExitApp

MenuUninstall:
  Gui, Submit
  If (bOption1 == 0 && bOption2 == 0) {
   Gui, Destroy
   GoTO L3_UserInterface
  }
; MsgBox, bOption1 = %bOption1%   bOption2 = %bOption2%
  UninstallFiles()
  UninstallMakerIni()
; ResetUserData()
  Finalise2()
ExitApp

; ===== detail functions =======================================================================================

PrepareFiles() { ; --------------------- Some files need adaptation depending on target -----------
; e.g. copy doc files into the script folder; prepare ini file etc.
; IniWrite, %FMlang%, %srcDir%\FMdict.ini, FMdict, language ; define UI-langauge for FMdict
; FileDelete,         %srcDir%\FMdict-log.txt  ; let the app start afresh
} ; --- end PrepareFiles

InstallFiles() { ; --------------------- Establish the program directory ---------------------------------------
  If (bOption1) {                                           ; install in StartUp folder
; msgBox, Option 1: %installDir%\scripts\%appName%*.jsx ➔ %FMautoRun%
    FileCopy,      %installDir%\scripts\%appName%.jsx, %FMautoRun%, 1       ; main script
    FileCopyDir,   %installDir%\scripts\%appName%, %FMautoRun%\%appName%, 1 ; scripts, pdf
    FileCopy,      %installDir%\scripts\FMxxx_OpenPdf.exe, %FMautoRun%, 1   ; direct access to pdf page
  }

  If (bOption2) {                                           ; install in MyDocuments
    If (!FileExist(FMscriptDir)) {
      FileCreateDir, %FMscriptDir%
    }
; msgBox, Option 2: %installDir%\scripts\%appName%*.jsx ➔ %FMscriptDir%
    FileCopy,      %installDir%\scripts\%appName%.jsx, %FMscriptDir%, 1       ; main script
    FileCopyDir,   %installDir%\scripts\%appName%, %FMscriptDir%\%appName%, 1 ; scripts, pdf
    FileCopy,      %installDir%\scripts\FMxxx_OpenPdf.exe, %FMscriptDir%, 1
  }
} ; --- end InstallFiles

ModifyMakerIni() { ; --- Modify FrameMaker settings in section [D+DD] -----------------------------
; IniWrite, Value, Filename, Section, Key
  IniWrite, v%appVersion%, %userIni%, D+DD, %appName%
  IniWrite, Citavi,                             %userIni%, D+DD, FMbib_BibAppName
  IniWrite, H:\Science\Citavi\bin\Citavi.exe,   %userIni%, D+DD, FMbib_BibAppLoc
  IniWrite, FMbiblio-collected.rtf,             %userIni%, D+DD, FMbib_CollectFile
  IniWrite, 0,                                  %userIni%, D+DD, FMbib_DoingBook
  IniWrite, 0,                                  %userIni%, D+DD, FMbib_Logging
  IniWrite, FMbiblio-tpl.fm,                    %userIni%, D+DD, FMbib_Template
} ; --- end ModifyMakerIni
/*
ModifyUserData() {                      ; specific for FMnotes
; get the location of the user data file to be amended (menus.cfg or etb-menus.cfg)
  menuDir := FMiniDir . "\WorkSpaces\UnStructured\WYSIWYGView\menus\"
; I:\UserData\Appdata\Roaming\Adobe\FrameMaker\15\WorkSpaces\UnStructured\WYSIWYGView\menus\etb-menus.cfg
  menuFile := menuDir . "etb-menus.cfg" ; also insert in menus.cfg
  menuText =
(
`n<Modify GotoPreviousView <KeySequence \!/Left>  <KeySeqLabel Escape «->>
<Modify GotoNextView     <KeySequence \!/Right> <KeySeqLabel Escape -»>>
)
; msgBox, menuFile = %menuFile%
  If (FileExist(menuFile)) {
    FileCopy %menuFile%, %menuFile%.saved, 1
    FileAppend %menuText%, %menuFile%
  }
  menuFile := menuDir . "menus.cfg"     ; also insert in menus.cfg
  If (FileExist(menuFile)) {
    FileCopy %menuFile%, %menuFile%.saved, 1
    FileAppend %menuText%, %menuFile%
  }
} ; --- end ModifyUserData

ResetUserData() {
  menuDir := FMiniDir . "\WorkSpaces\UnStructured\WYSIWYGView\menus\"
  menuFile := menuDir . "etb-menus.cfg"
  If (FileExist(menuFile . ".saved")) {
    FileMove %menuFile%.saved, %menuFile%, 1
  }
  menuFile := menuDir . "menus.cfg"
  If (FileExist(menuFile . ".saved")) {
    FileMove %menuFile%.saved, %menuFile%, 1
  }
} ; --- end ResetUserData
*/
UnInstallFiles() { ; --- Uninstall all files installed by this routine ----------------------------
  If (FileExist(FMautoRun . "\" . appName . ".*")) {
    FileDelete,    %FMautoRun%\%appName%.*                   ; main script
    FileRemoveDir, %FMautoRun%\%appName%, 1                  ; scripts, pdf
;   FMxxx_OpenPdf.exe may still be used by other scripts
  }
  If (FileExist(FMscriptDir)) {
    FileRemoveDir, %FMscriptDir%, 1                         ; MyDocuments: dir and files
;   FMxxx_OpenPdf.exe may still be used by other scripts
  }
; Lines at the bottom of the menu file are not removed (to great a hassle)
} ; --- end UnInstallFiles

UnInstallMakerIni() {
; --- Remove items from D+DD ini section - section may be used for other projects also
  IniDelete, %userIni%, D+DD, %appName%
  IniDelete, %userIni%, D+DD, FMbib_BibAppName
  IniDelete, %userIni%, D+DD, FMbib_BibAppLoc
  IniDelete, %userIni%, D+DD, FMbib_CollectFile
  IniDelete, %userIni%, D+DD, FMbib_DoingBook
  IniDelete, %userIni%, D+DD, FMbib_Logging
  IniDelete, %userIni%, D+DD, FMbib_Template
} ; --- end UnInstallFiles

CreateShortCut() { ; ---------------------- Create shortcut to installation-pgm -------------------
  If (!FileExist(shortCutDir)) {
		FileCreateDir, %shortCutDir%
	}
	FileCreateShortcut, %installDir%\Inst-%appName%.exe, %shortCutDir%\%appTitle%.lnk, , , %appTitle%, %iconPath%
; --------------------------------------------------------- Create shortcut for the utility
; FileCreateShortCut, %installDir%\scripts\%appName%.exe, %shortCutDir%\%appName%.lnk, , , , %iconPath%
} ; --- end CreateShortCut

Finalize() { ; --- Installation: Remove Installation Preparation files ----------------------------
  RemoveInstFiles()
  if (FMlang == "en" ) {
  	MsgBox, 64, %title%, %appName% installed succesfully.
  } else if (FMlang == "de" ) {
  	MsgBox, 64, %title%, %appName% wurde installiert.
  } else if (FMlang == "fr" ) {
  	MsgBox, 64, %title%, %appName% a été installé.
  }
} ; --- end Finalise

Finalise2() { ; --- De-Installation: Remove Installation Preparation files ------------------------
  RemoveInstFiles()
  if (FMlang == "en" ) {
  	MsgBox, 64, %appTitle%, %appName% succesfully uninstalled.
  } else if (FMlang == "de" ) {
  	MsgBox, 64, %appTitle%, %appName% wurde de-installiert.
  } else if (FMlang == "fr" ) {
  	MsgBox, 64, %appTitle%, %appName% a été de-installé.
  }
} ; --- end Finalise2

RemoveInstFiles() { ; --- Remove Installation Preparation files -----------------------------------
  SplitPath, auxDir, , , , , OutDrive
  StringUpper, OutDrive, OutDrive
; MsgBox,  auxDir = %auxDir%  `n  OutDrive = %OutDrive%
  If (OutDrive == "C:") {                                   ; protect development folders from deletion
	If (FileExist(auxDir . "\InstallThis")) {                 ; be aware of second install
		FileRemoveDir, %auxDir%\InstallThis, 1
		if ErrorLevel {
			MsgNoAccess(FMlang)
			ExitApp
		}
		FileDelete, %auxDir%\PrepareInstallation.exe
		ClearDesktop()                                          ; assure removal of icons
		}
  }
} ; --- end RemoveInstFiles

FunGetFMprops(FMtarget) { ; --- Get some properties of the FM-installation ------------------------
; Global      FMpgm (full path), FMlang (en|de|fr), FMfullVersion (14.0.2.425), FMmarket (2017)
; Arguments   FMtarget: number of requested FM version (9 or 15 or "9" or "15"=
;                       If empty string, then the current version is searched
; Returns:    Path for FMhome (e.g. H:\Adobe\FrameMaker.15en64\Adobe FrameMaker 2018)
;             Terminating \ is removed for compatibility
  FMroot := FunGetFMlocation(FMtarget)
  FMpgm  := FMroot . "\FrameMaker.exe"
  FileGetVersion,  FMfullVersion, %FMpgm%
  FMversion   := SubStr(FMfullversion, 1, 2)
  FMlang := "en"                        ; determine the UI language
  IniRead, FMlanguage, %FMroot%\maker.ini, Frame, Language, UKEnglish
  If InStr(FMlanguage, "French") {
    FMlang := "fr"
  } Else If InStr(FMlanguage, "German") {
    FMlang := "de"
  }
  Return FMroot
} ; --- end FunGetFMprops

FunGetFMlocation(FMtarget) { ; --- Get FMhome directory of particular or current FM installation -----
; Arguments   FMtarget: number of requested FM version (9 or 15 or "9" or "15"=
;                       If empty string, then the current version is searched
; Returns;    Path for FMhome (e.g. H:\Adobe\FrameMaker.15en64\Adobe FrameMaker 2018)
;             Terminating \ is removed for compatibility
; Comment;    64 bit apps write the registry entries in HKEY_LOCAL_MACHINE\SOFTWARE\Adobe\FrameMaker
;             32 bit apps write the registry entries in WOW mode
  If (A_Is64bitOS) {
    SetRegView, 64                      ; view registry as 64 bit app - AHK is 32 bit app
  }
  If (FMtarget != "") {                                     ; search for a specific version of FM
  	RegRead, fmRoot, HKEY_LOCAL_MACHINE\SOFTWARE\Adobe\FrameMaker\%FMtarget%, FMHome
    If (fmRoot == "" ) {
  	  RegRead, fmRoot, HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Adobe\FrameMaker\%FMtarget%, FMHome
    }
    If (fmRoot == "" ) {                ; FM version can not be found
      Return ""
    } Else {
      If (SubStr(fmRoot, 0) == "\") {
        StringTrimRight, fmRoot, fmRoot, 1 ; remove traing \
      }
      Return %fmRoot%
    }
  } Else {                              ; search for the currently active FM version (double click on FM file)
    RegRead, docType, HKEY_CLASSES_ROOT\.fm          ; Default
    RegRead, command, HKEY_CLASSES_ROOT\%docType%\shell\Open\command  ; Default
    If (command == "" ) {               ; FM version can not be found
      Return ""
    } Else {
      StringTrimRight, fmRoot, command, 21 ; remove trailng \FrameMaker.exe" "%1"
      StringTrimLeft,  fmRoot, fmRoot, 1   ; remove initial "
      Return fmRoot
    }
  }
} ; --- end FunGetFMlocation

GetCfgData() { ; --- Get data from cfg file -------------------------------------------------------
  IniRead, appDescr,   %cfgFile%, %FMlang%, appDescr
; IniRead, appHint,    %cfgFile%, %FMlang%, appHint
  IniRead, appIcon,    %cfgFile%, Project,  appIcon         ; FM-puzzle.ico
  IniRead, appName,    %cfgFile%, Project,  appName         ; FMvars
  IniRead, appTitle,   %cfgFile%, %FMlang%, appTitle
  IniRead, appVersion, %cfgFile%, Project,  appVersion      ; 1.2
  IniRead, auxDir,     %cfgFile%, Project,  auxDir
  IniRead, FMtarget,   %cfgFile%, Project,  FMtarget
  IniRead, helpFile,   %cfgFile%, Project,  helpFile        ; fmaker79.html
  IniRead, instHint,   %cfgFile%, %FMlang%, instHint
  IniRead, option1,    %cfgFile%, %FMlang%, option1
  IniRead, option2,    %cfgFile%, %FMlang%, option2
  IniRead, btnHelp,    %cfgFile%, %FMlang%, btnHelp
  IniRead, btnInstall, %cfgFile%, %FMlang%, btnInstall
  IniRead, btnUnInst,  %cfgFile%, %FMlang%, btnUnInst
  IniRead, btnCancel,  %cfgFile%, %FMlang%, btnCancel
} ; --- end GetCfgData

ClearDesktop() { ; --- Clear Desktop --------------------------------------------------------------
  Sleep, 2000                                               ; assure last actions
  WinActivate, ahk_class Progman
  Send, {F5}                                                ; refresh desktop to make icon disappear
} ; --- end ClearDesktop

; === Messages ====================================================================================

MsgNoAccess(language) {
  If (language == "de") {
  	MsgBox, 48, %appTitle%,
    (
    Nicht mehr benötigte Dateien aus der Installations-Vorbereitung
    konnten nicht gelöscht werden (vermutlich Zugriffsverweigerung).

    Bitten Sie ihren Systemadministrator, diese Überbleibsel zu entfernen
    [sie sind harmlos fürs System, es geht nur ums Aufräumen].
    )
  } Else If (language = "fr") {
    MsgBox, 48, %appTitle%,
    (
    Les fichiers plus utilisés de la Préparation à l'Installation
    n'a pas pu être supprimé (restrictions d'accès possibles).

    S'il vous plaît laissez votre adiministrateur supprimer ces vestiges
    [Ils ne nuisent pas à votre système, c'est juste un nettoyage].
    )
  } Else {
    MsgBox, 48, %appTitle%,
    (
    The no more used files from the Installation Preparation
    could not be deleted (propably access restrictions)

    Please let Your adiministrator remove these left overs
    [they do not harm your system, it's just a cleaning up]
    )
  }
} ; --- end MsgNoAccess
