﻿/* FMbiblio.jsx ------ UTF-8 ----------------------------------------------------------------------
                Insert Bibliography into FrameMakaer with Citavi/EndNote …
  Documentation www.daube.ch/docu/fmaker70.html
  Comment       To be placed in Startup folder of user area
                PrepareInstallation is assumed to be on desktop together with project installation-directory
                This installation program resides in \InstallThis (1 level below PrepareInstallation)
                This utility program resides in \InstallThis\scripts (2 levels below PrepareInstallation)
  Statistics    as of 2018-09-28                  2021-05-25
                Number of Modules =	1             4 (jsx) + 3 (xml)
                Number of Functions =	55          56
                Number of pgm lines =	726
  Attention     In FM the TAB has code \x08, not \x09 as in ASCII
  Tasks         • The utility script FMbiblio resides in \InstallThis\cripts and may be ahk and/or jsx
                • Application scripts will be copied into the FM-user-area and optionally the Adobe script folder
  Reference     See "Knowledge and information sources" in documentation
                Traduction français par Emmanuèle Rüegger († 2017-10-05)
  AHK version   1.1.29.00
                Assure that AutoHotkeySC.bin is the same as  Unicode 32-bit.bin
                Only this way the compiled scripts run on both x64 and x32 systems
  Copyleft      2018 Klaus Daube, Zürich, Switzerland, klaus@daube.ch
  History       2016-02-02 v1.0 released:
                2017-05-03 v1.1 Coding error in German and French language files corrected
                2018-03-15 v1.2 New project structure, new command names
                2018-09-24 v1.3 problem determination and correction. See website/Developer-log.fm
                           Handle error when reading RTF; fully comment functions
                2021-05-11 Start converstion to DDD coding standard, localisation by XML files
                2021-05-26 publishing v2.0
*/ ; // ===========================================================================================
#target framemaker

if (typeof KLD_B == "undefined") {
  KLD_B = {};                                     // Global script object for this suite
}
#include FMbiblio\FMbiblio_Globals.jsx            // all declarations, includes FMppp_Texts.jsx
#include FMbiblio\FMbiblio_Ancillary.jsx          // ancillary functions
#include FMbiblio\FMbiblio_DocObj.jsx             // document related functions
//       FMbiblio\FMbiblio_en.xml   // Read by SwitchUIlanguageX
//       FMbiblio\FMbiblio_de.xml   // See there
//       FMbiblio\FMbiblio_fr.xml   //

KLD_B.Main = function () { // =====================================================================
/*
Called by     system
Calling       FMbiblioSetup, ReportInfo, SetupMenus, Debugmenu
// --- For debugging you need to switch statements (ESTK not working when menus active)
                         real work       debugging
*/
var bIsOK = false;
  KLD_B.FM_VERSION = app.VersionMajor;
  Console (KLD_B.APP_NAME + " v" + KLD_B.APP_VERSION);
  KLD_B.APP_LANG = KLD_B.SwitchUIlanguageX (KLD_B.APP_NAME); // etablish the UI texts
  bIsOK = KLD_B.FMbiblioSetup (); // get data from the ini file -- debug to be set explicitly
  if (!bIsOK) {return false;}
// --- For debugging you need to switch statements (ESTK not working when menus active)
//                                   real work    debugging
  KLD_B.SetUpMenus ();            // uncomment    comment
//KLD_B.DebugMenu ();             // Comment      uncomment
  if (KLD_B.bLogging) {KLD_B.ReportInfo ()};    // info about this run
} //--- end Main ----------------------------------------------------------------------------------

KLD_B.DebugMenu = function () { // ================================================================
/*            Ancillary menu during debugging with ESTK or VScode
Called by     main
Calling       Command, FillItemList
Comment       Used only during development - not invoked in productive version
*/
var aMenuItems, listIndex, txtIntro, wDlgDebug;

  aMenuItems = [
  "Documentation",                                // 0
  "1 Collect temporary citations",                // 1
  "2 Call bibliographic application",             // 2
  "3a Resolve temporary citations",               // 3
  "3b Expand temporary citations in footnotes"]   // 4

  wDlgDebug = new Window("dialog","Debugging Menu",undefined);
  wDlgDebug.location= [200,200];
  wDlgDebug.t1 = wDlgDebug.add ("statictext", undefined, "Select menu function to be executed with DOUBLE Click.", {multiline:true});
  wDlgDebug.t1.preferredSize.width = 400;
	wDlgDebug.g1 = wDlgDebug.add ("group", undefined, undefined);
  	wDlgDebug.g1.l1 = wDlgDebug.g1.add ("listbox", undefined, undefined);
    wDlgDebug.g1.l1.preferredSize = [350,100];
  KLD_B.FillItemList (wDlgDebug.g1.l1, aMenuItems);

  wDlgDebug.g1.l1.onDoubleClick = function () {
    listIndex = wDlgDebug.g1.l1.selection.valueOf();
    Command (listIndex + 1);
  };

  wDlgDebug.show();
} //--- end DebugMenu ------------------------------------------------------------------------------

KLD_B.SetUpMenus = function () { // ===============================================================
/*
Called by     main
Calling       Command, UpdateMenus
Attention T   his function MUST NOT be named DefineMenu - because this is a reserved function!
Comment   T   here is no menu "Special" in books => FormatMenu used instead for both cases
          F   or a list of DDD used Esc shortcuts see E:\_DDDprojects\FM-GUI-modifications\ESC-sequences\FM-ESC-sequences.fm
Reference K   laus Göbel at https://forums.adobe.com/thread/2236201
             (My method with kb-labels after \t in the menu entry defintions does not work in FM-14)
*/
var bkFMbiblioMenu, docFMbiblioMenu, menuLocation, oMenus = {}, oCmd = {};
// --- Menu entries (could be integrated in to command definitions as strings)
  oMenus.MenuMain   = KLD_B.UItxt.SetUpMenus.@menu00 + " v" + KLD_B.APP_VERSION;;
  oMenus.MenuDocu   = KLD_B.UItxt.SetUpMenus.@menu01;
  oMenus.MenuColl   = KLD_B.UItxt.SetUpMenus.@menu02;
  oMenus.MenuBibapp = KLD_B.UItxt.SetUpMenus.@menu03;
  oMenus.MenuRes1   = KLD_B.UItxt.SetUpMenus.@menu04;
  oMenus.MenuRes2   = KLD_B.UItxt.SetUpMenus.@menu05;
// --- Command definition for documents and books
  oCmd.MenuDocu   = DefineCommand(1,"FMbiblio_Docu",    oMenus.MenuDocu, "");
  oCmd.MenuColl   = DefineCommand(2,"FMbiblio_Collect", oMenus.MenuColl, "");
  oCmd.MenuBibapp = DefineCommand(3,"FMbiblio_Bibapp",  oMenus.MenuBibapp, "");
  oCmd.MenuRes1   = DefineCommand(4,"FMbiblio_Resolve", oMenus.MenuRes1, "");
  oCmd.MenuRes2   = DefineCommand(5,"FMbiblio_Expand",  oMenus.MenuRes2, "");
// --- Shortcut labels - none
// --- Assign commands to menus
  menuLocation = app.GetNamedMenu("FormatMenu");
  docFMbiblioMenu  = menuLocation.DefineAndAddMenu("!FMbiblio_main", oMenus.MenuMain);   // ! required
  docFMbiblioMenu.AddCommandToMenu (oCmd.MenuDocu);
  docFMbiblioMenu.AddCommandToMenu (oCmd.MenuColl);
  docFMbiblioMenu.AddCommandToMenu (oCmd.MenuBibapp);
  docFMbiblioMenu.AddCommandToMenu (oCmd.MenuRes1);
  docFMbiblioMenu.AddCommandToMenu (oCmd.MenuRes2);
  menuLocation = app.GetNamedMenu("BookFormatMenu");
  bkFMbiblioMenu  = menuLocation.DefineAndAddMenu("!bkFMbiblioMain", oMenus.MenuMain); // ! required
  bkFMbiblioMenu.AddCommandToMenu (oCmd.MenuDocu);
  bkFMbiblioMenu.AddCommandToMenu (oCmd.MenuColl);
  bkFMbiblioMenu.AddCommandToMenu (oCmd.MenuBibapp);
  bkFMbiblioMenu.AddCommandToMenu (oCmd.MenuRes1);
  bkFMbiblioMenu.AddCommandToMenu (oCmd.MenuRes2);

  UpdateMenus();                                  // refresh and actually make the new menu appear
} // --- end SetUpMenus ---------------------------------------------------------------------------

function Command (cmd) { // =======================================================================
/*
Arguments     cmd:  command number to be executed
Called by     DebugMenu, SetUpMenus
Calling       OpenHelpFile, FMbiblioCollect, FMbiblioCallBibapp, FMbiblioResolve, FMbiblioExpand
*/
  switch (cmd) {
    case 1:
      KLD_B.OpenHelpFile ();
      break;
    case 2:
      KLD_B.FMbiblioCollect ();
      break;
    case 3:
      KLD_B.FMbiblioCallBibapp ();
      break;
    case 4:
      KLD_B.FMbiblioResolve ();
      break;
    case 5:
      KLD_B.FMbiblioExpand ();
      break;
  }
} // --- end Command ------------------------------------------------------------------------------

KLD_B.FMbiblioCollect = function () { // ==========================================================
/*            Collect temporary citations
Called by     Command
Calling       IconMessage, MakerIniWrite, GetDocPath, CollectInBook, CollectInDoc, ReportFoundCitations, CreateBibDoc, SaveFileRTF
*/
var docFile, newDoc;

  if ((app.ActiveDoc.ObjectValid() === 0)  && (app.ActiveBook.ObjectValid() === 0)){
    KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_10, "FMbiblioCollect");   // please open document or book
    return;
  }
  KLD_B.asFndCitations = [];                      // clear array from previous run
  if (app.ActiveBook.ObjectValid()) {
    docFile = app.ActiveBook.Name;
    KLD_B.bDoingBook = true;
    KLD_B.MakerIniWrite ("D+DD", "FMbib_DoingBook = yes");
    if (KLD_B.bLogging) {KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_22, "FMbiblioCollect", docFile);}
    KLD_B.sCollectPath = KLD_B.GetDocPath ();     // path of active book
    KLD_B.CollectInBook (app.ActiveBook);
  } else {
    docFile = app.ActiveDoc.Name;
    KLD_B.bDoingBook = false;
    KLD_B.MakerIniWrite ("D+DD", "FMbib_DoingBook = no");
    if (KLD_B.bLogging) {KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_23, "FMbiblioCollect", docFile);}
    KLD_B.sCollectPath = KLD_B.GetDocPath ();     // path of active document
    KLD_B.CollectInDoc (app.ActiveDoc);
  }
// --- Insert found citation in new doc and save it as RTF
  if (KLD_B.asFndCitations.length === 0) {        // no citations found in doc or book
    KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_13, "FMbiblioCollect");
    return;
  }
  if (KLD_B.bLogging) {KLD_B.ReportFoundCitations ();}
  newDoc = KLD_B.CreateBibDoc ();                 // write citations to RTF
  KLD_B.SaveFileRTF (newDoc, KLD_B.sCollectFile, KLD_B.sCollectPath);
  KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_25, "FMbiblioCollect");
} // --- end FMbiblioCollect ----------------------------------------------------------------------

KLD_B.FMbiblioCallBibapp = function () { // =======================================================
/*            Call bibliographic application
Called by     Command
Calling       Message, GetDocPath, CopyToClipboard
*/
var msgTxt, bibAppPgm;
  if (KLD_B.sBibAppName.toLowerCase() === "citavi") {
    msgTxt = KLD_B.UItxt.Function.@msg_14;                                   // Citavi
  } else {
    msgTxt  = KLD_B.UItxt.Function.@msg_15;                                  // EndNote
  }
  bibAppPgm = File (KLD_B.sBibAppLoc);
  if (bibAppPgm.exists) {
    KLD_B.IconMessage ("I", msgTxt, "FMbiblioCallBibapp", KLD_B.sBibAppName); // bibliographic application ... startingd
    KLD_B.sCollectPath = KLD_B.GetDocPath ();                // path for active document
    KLD_B.CopyToClipboard (KLD_B.sCollectPath + "\\" + KLD_B.sCollectFile);
    bibAppPgm.execute();
  } else {                                        // The bibliographic application %01 not found at %02
    KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_16, "FMbiblioCallBibapp", KLD_B.sBibAppName, KLD_B.sBibAppLoc);
    return;
  }
} // --- end FMbiblioCallBibapp -------------------------------------------------------------------

KLD_B.FMbiblioResolve = function () { // ==========================================================
/*            Resolve temporary citations
Called by     Command
Calling       GetDocPath, Message, ReadFileRTF, IsBookRequiredQ, SetDisplay, ReplaceInBook, ReplaceInDoc, ReportResolvedCitations, RefreshDisplay
*/
var inputFile, prompt, docFile;
  KLD_B.asFmtCitsRaw.length = 0;                  // Arrays may be filled by previous use
  KLD_B.asFmtCitsFmt.length = 0;
  KLD_B.aoBibliography.length = 0;
  KLD_B.iReplCount = 0;

  prompt = KLD_B.sIniTexts [18];                  // --- Select the RTF file with the formatted citations
  inputFile = ChooseFile (prompt, KLD_B.GetDocPath (), "*.rtf", Constants.FV_ChooseSelect);
  if (inputFile === null) {
    KLD_B.IconMessage ("E", KLD_B.UItxt.Function.@msg_20, "FMbiblioResolve"); // --- File selection cancelled, step aborted
    return;
  }
  if (!KLD_B.ReadFileRTF (inputFile)) {           // get the modified RTF file and fill global arrays
    KLD_B.RefreshDisplay ();                      // an error occured in ReadFileRTF
    return;
  }                                               // the last opened doc is active now
  if ( KLD_B.IsBookRequiredQ ()) {                // did we collect in a book or a document ?
    if (app.FirstOpenBook.ObjectValid()){
      app.ActiveBook = app.FirstOpenBook;         // go back to the book window
    } else {
      KLD_B.IconMessage ("E", KLD_B.UItxt.Function.@msg_17, "FMbiblioResolve"); // --- collected from book, but it's not active
      return;
    }
  }
  KLD_B.SetDisplay (false);                       // avoid screen flicker
  if (app.ActiveBook.ObjectValid()) {             // replace in book
    if (KLD_B.bLogging) {KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_22, "FMbiblioResolve", app.ActiveBook.Name);}
    KLD_B.ReplaceInBook (app.ActiveBook);
  } else {                                        // replace in document
    if (KLD_B.bLogging) {KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_23, "FMbiblioResolve", app.ActiveDoc.Name);}
    KLD_B.ReplaceInDoc (app.ActiveDoc);
  }
  KLD_B.SetDisplay (true);                        // visibility on again
  if (KLD_B.bLogging) {KLD_B.ReportResolvedCitations (KLD_B.iCitUnique, KLD_B.iCitUnResolved, KLD_B.iBibEntries)};
  KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_21, "FMbiblioResolve", KLD_B.iCitUnique, KLD_B.iCitUnResolved, KLD_B.iBibEntries, KLD_B.iReplCount);
  KLD_B.RefreshDisplay ();                        // update document display
} // --- end FMbiblioResolve ----------------------------------------------------------------------

KLD_B.FMbiblioExpand = function () { // ===========================================================
/*            Expand temporary citations in footnotes
Called by     Command
Calling       GetDocPath, Message, GetBiblioFromRTF, IsBookRequiredQ, SetDisplay, ExpandInBook,
              ExpandInDoc, ReportResolvedCitations, RefreshDisplay
*/
var inputFile, prompt,
    sourceDoc,                                    // source document of the replacements in the array
    targetDoc;                                    // target document where to replace
  KLD_B.asFmtCitsRaw.length = 0;                  // Arrays may be filled by previous use
  KLD_B.asFmtCitsFmt.length = 0;
  KLD_B.aoBibliography.length = 0;
  KLD_B.iReplCount = 0;

  prompt = KLD_B.sIniTexts [18];                  // --- Select the RTF file with the formatted citations
  inputFile = ChooseFile (prompt, KLD_B.GetDocPath (), "*.rtf", Constants.FV_ChooseSelect);
  if (inputFile === null) {
    KLD_B.IconMessage ("E", KLD_B.UItxt.Function.@msg_20, "FMbiblioExpand"); // --- File selection cancelled, step aborted
    return;
  }
  KLD_B.oCurrentDoc = app.ActiveDoc;              // save 'calling' document
  if (!KLD_B.GetBiblioFromRTF (inputFile)) {      // get the modified RTF file and fill global arrays
    KLD_B.RefreshDisplay ();                      // an error occured in GetBiblioFromRTF
    return;
  }
  targetDoc = KLD_B.oCurrentDoc;                  // back to user document (= app.ActiveDoc)
  sourceDoc = KLD_B.oRtfDoc;                      // required to access items via arrays
  if ( KLD_B.IsBookRequiredQ ()) {                // did we collect in a book or a document ?
    if (app.FirstOpenBook.ObjectValid()){         // go back to the book window
      app.ActiveBook = app.FirstOpenBook;
    } else {
      KLD_B.IconMessage ("E", KLD_B.UItxt.Function.@msg_17, "FMbiblioResolve"); // --- collected from book, but it's not active
      return;
    }
  }

  KLD_B.SetDisplay (false);                       // avoid screen flicker
  if (app.ActiveBook.ObjectValid()) {             // replace in book
    KLD_B.ExpandInBook (app.ActiveBook, sourceDoc);
  } else {                                        // replace in document
    KLD_B.ExpandInDoc (targetDoc, sourceDoc);
  }
  if (KLD_B.oRtfDoc.ObjectValid()) {              // may have been closed already
    KLD_B.oRtfDoc.Close (Constants.FF_CLOSE_MODIFIED); // had to be postponed until now
  }

  KLD_B.SetDisplay (true);                        // visibility on again

  if (KLD_B.bLogging) {KLD_B.ReportResolvedCitations (KLD_B.iCitUnique, KLD_B.iCitUnResolved, KLD_B.iBibEntries)};
  KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_21, "FMbiblioResolve", KLD_B.iCitUnique, KLD_B.iCitUnResolved, KLD_B.iBibEntries, KLD_B.iReplCount);
  KLD_B.RefreshDisplay ();                        // update document display
} // --- end FMbiblioExpand -----------------------------------------------------------------------

KLD_B.FMbiblioSetup = function () { // ============================================================
/*            Set up script environment
Called by     Main
Calling       MakerIniRead, SwitchUIlanguageX
*/
var asSettings = [], sHelpFile;

  KLD_B.sScriptDir = KLD_B.GetScriptPath ();
  sHelpFile        = KLD_B.sScriptDir + "\\" + KLD_B.APP_NAME + ".pdf";
  KLD_B.oHelpFile  = File (sHelpFile);
  KLD_B.sIniFile   = KLD_B.sScriptDir + "\\" +  "FMbiblio.ini";

  asSettings = KLD_B.MakerIniRead ("D+DD", "FMbib_BibAppName", "FMbib_BibAppLoc", "FMbib_CollectFile", "FMbib_DoingBook", "FMbib_Logging", "FMbib_Template");          // Get the initial settings ---------------------

  KLD_B.sBibAppName  = asSettings[0];             // name of the bibliographic application
  KLD_B.sBibAppLoc   = asSettings[1];             // path where gsFMbib_BibAppName is located
  KLD_B.sCollectFile = asSettings[2];             // file name; collected items
  KLD_B.bDoingBook   = KLD_B.StringToBool(asSettings[3]); // Whether started in a book
  KLD_B.bLogging     = KLD_B.StringToBool(asSettings[4]); // Log bibliographic data
  KLD_B.sTemplate    = asSettings[5];             // located in script-dir
  return true;
} // --- end FMbiblioSetup ------------------------------------------------------------------------

KLD_B.ExpandInDoc = function (targetDoc, sourceDoc) { // ==========================================
/*            Expand temporary citation in document
Parameter     targetDoc: target document where to replace
              sourceDoc: source document of the replacements in the array
Called by     FMbiblioExpand, ExpandInBook
Calling       FindAndReplacePara, Message
*/
var j, jBiblio, parts = [], nBiblio, nCitations, findString, trReplace, replacementCounter, loopMax, overallCounter = 0,
    nCitations = KLD_B.asFmtCitsRaw.length,
    loopMax = nCitations;

  pgf = targetDoc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;  // get first pgf in flow

  for (j = 0; j < nCitations; j += 1) {
    findString    = KLD_B.asFmtCitsRaw[j];        // there are no 'undefined' in here
    jBiblio       = KLD_B.asFmtCitsFmt[j];        // number (or list of) biblio-paragraph
// From combined citations only the first can be handled (after first replacement findString no more exists ins doc)
    parts = jBiblio.split(";");                   // only semicolon is produced, no other punctuation
    trReplace = KLD_B.aoBibliography[parts[0]-1]; // is object TextRange
    replacementCounter = KLD_B.FindAndReplacePara (targetDoc, findString, sourceDoc, trReplace, loopMax);
    overallCounter += replacementCounter;
  }
  if (KLD_B.bLogging) {KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_24, "ExpandInDoc", overallCounter);}
  KLD_B.iReplCount += overallCounter;
} // --- end ExpandInDoc --------------------------------------------------------------------------

KLD_B.ExpandInBook = function (book, sourceDoc) { // ==============================================
/*            Expand temporary citation in book
Arguments     book:      book object to be handled
              sourceDoc: source file from which the formatted citations come
Called by     FMbiblioExpand
Calling       GetOpenParams, ExpandInDoc
Comment       doc = SimpleOpen (docName, false) Does not handle open errors automatically
*/
var targetDoc, docName, bookComp = book.FirstComponentInBook;
  while(bookComp.ObjectValid()) {                 // process only FM docs
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) {
      book.StatusLine = "Processing " + File(bookComp.Name).displayName;
      docName = bookComp.Name;
      openParams = KLD_B.GetOpenParams ();
      openReturnParams =  new PropVals();
      targetDoc = Open (docName, openParams, openReturnParams);
      KLD_B.ExpandInDoc (targetDoc, sourceDoc);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
  book.StatusLine = "";
} // --- end ExpandInBook -------------------------------------------------------------------------

KLD_B.CreateBibDoc = function () { // =============================================================
/*            Create document with bibliography
Called by     FMbiblioCollect
Calling       GetDocPath, SortUnique, OpenTemplate
returns       created doc in function
*/
var j, uniqueCitations= [], nUnique, nItems, insText, newDoc, pgf, textloc, j,
    writeFile = KLD_B.GetDocPath () + KLD_B.sCollectFile;

// create array of unique citations and place them in the doc
  uniqueCitations = KLD_B.SortUnique (KLD_B.asFndCitations);
  nUnique= uniqueCitations.length;
// createa new file
  newDoc = KLD_B.OpenTemplate ();                 // FM-10/11/15 undefined is not an object. FM-12/13 OK
  pgf = newDoc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;  // get first pgf in flow
  for (j = 0; j < nUnique; j += 1) {
    textLoc = new TextLoc (pgf, 0);
    insText = uniqueCitations[j];
    newDoc.AddText (textLoc, "[["+insText+"]]"+ "\x08" + "{"+insText+"}"); // insert FM TAB
   if (j < nUnique -1){
    pgf = newDoc.NewSeriesPgf(pgf);               // add a paragraph
    }
  }
  return newDoc;
} // --- end CreateBibDoc -------------------------------------------------------------------------

KLD_B.OpenHelpFile = function () { // =============================================================
/*            Menu item "Documentation"
              Display the pdf with same name as this script ($.filename includes extension)
Called by     Command
Comment       No feedback in case the pdf does not exist => probably to be changed
*/
  if (KLD_B.oHelpFile.exists) {
    KLD_B.oHelpFile.execute();
  }
} // --- end OpenHelpfFile ------------------------------------------------------------------------

KLD_B.ReplaceInBook = function (book) { // ========================================================
/*
Parameter b   ook: book object to be handled
Called by     FMbiblioResolve
Calling       GetOpenParams, ReplaceInDoc
Comment       Documents may be open or closed
              doc = SimpleOpen (docName, false) Does not handle open errors automatically
*/
var oDoc, docNme, bookComp = book.FirstComponentInBook;
  while(bookComp.ObjectValid()) {                 // process only FM docs
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) {
      book.StatusLine = "Processing " + File(bookComp.Name).displayName;
      docName = bookComp.Name;
      openParams = KLD_B.GetOpenParams ();
      openReturnParams =  new PropVals();
      oDoc = Open (docName, openParams, openReturnParams);
      KLD_B.ReplaceInDoc (oDoc);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
  book.StatusLine = "";
} // --- end ReplaceInBook ------------------------------------------------------------------------

KLD_B.ReplaceInDoc = function (oDoc) { // =========================================================
/*
Arguments     oDoc: document object to be handled
Called by     FMbiblioResolve, ReplaceInBook
Calling       FindAndReplaceString, Message
Commment  D   ocument is open already
*/
var j, nBiblio, findString, replaceString, textLoc, replacementCounter, overallCounter = 0,
    nCitations = KLD_B.asFmtCitsRaw.length, loopMax = nCitations;

  pgf = oDoc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;  // get first pgf in flow
  textLoc = new TextLoc (pgf, 0);                 // start at the beginning of first para

  for (j = 0; j < nCitations; j += 1) {
    findString    = KLD_B.asFmtCitsRaw[j];
    replaceString = KLD_B.asFmtCitsFmt[j];             // unresolved have been filtered out in ReadFileRTF
    replacementCounter = KLD_B.FindAndReplaceString (oDoc, findString, replaceString, loopMax);
    overallCounter += replacementCounter;
  }
  if (KLD_B.bLogging) {KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_24, "ReplaceInDoc", overallCounter);}
  KLD_B.iReplCount += overallCounter;
} // --- end ReplaceInDoc -------------------------------------------------------------------------

KLD_B.ReportFoundCitations = function () { // =====================================================
/*
Called by     FMbiblioCollect
Calling       SortUnique
Comment       this is used only in debug
*/
var j, nCitations, uniqueCitations, nUnique;
  nCitations = KLD_B.asFndCitations.length;
  Console ("Citations found: " + nCitations + " ==========");
  for (j = 0; j < nCitations; j += 1) {
    Console (KLD_B.asFndCitations[j]);
  }
  uniqueCitations = [];
  uniqueCitations = KLD_B.SortUnique (KLD_B.asFndCitations);
  nUnique= uniqueCitations.length;
  Console ("\nUnique citations found: " + nUnique + " ----------");
  for (j = 0; j < nUnique; j += 1) {
    Console (uniqueCitations[j]);
  }
  Console ("\n ========== end of report ==========");
} // --- end ReportFoundCitations -----------------------------------------------------------------

KLD_B.ReportInfo = function () { // ===============================================================
/*
Called by     main
Calling       GetScriptPath, GetDocPath
*/
//  Console ("Logging for \t" + KLD_B.sAPP_NAME + " " + KLD_B.sAPP_VERSION);
    Console ("ScriptDir = \t" + KLD_B.GetScriptPath () );
    if ((app.ActiveDoc.ObjectValid() !== 0)  || (app.ActiveBook.ObjectValid() !== 0)){
      Console ("Book/DocPath = \t" + KLD_B.GetDocPath () ); // will be checked again
    } else {
      Console ("Book/DocPath = \t-- no book/document open --");
    }
    Console ("Doing a book = \t" + KLD_B.bDoingBook);
    Console ("Template file = \t" + KLD_B.sTemplate);
    Console ("Collect file = \t" + KLD_B.sCollectFile);
    Console ("Biblographic app = \t" + KLD_B.sBibAppName + " (" + KLD_B.sBibAppLoc + ")");
    Console ("\n");
} // --- end ReportInfo ---------------------------------------------------------------------------

KLD_B.ReportResolvedCitations = function (nCits, nUnres, nBibs) { // ==============================
/*
Arguments     nCits:   number of "unique" citations
              nUnres:  number of unresolved temp. citations
              nBibs:   number of bibliographic entries in second part of formatted RTF
Called by     FMbiblioResolve, FMbiblioExpand
Calling       -
Comment       Used only in verbose debugging
*/
var j;
  Console ("\n ========== Report Resolved Citations ==========");
  Console ("\n   Unresolved citations: " + nUnres);
  Console ("\n   Unique citations: " + nCits);
  for (j = 0; j < nCits; j += 1) {
    Console (KLD_B.asFmtCitsRaw[j] + "\t\t" + KLD_B.asFmtCitsFmt[j]);
  }
  Console ("\nBibliography: " + nBibs + " ----------");
  for (j = 0; j < nBibs; j += 1) {
    Console (KLD_B.aoBibliography[j]);
  }
  Console ("\n ========== end of report ==========");
} // --- end ReportResolvedCitations --------------------------------------------------------------

// <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>
KLD_B.Main ();                                          // follow the standard …
