﻿// --- FMbiblio_Ancillary.jsx ------ UTF-8 --------------------------------------------------------
// to be included in FMbiblio.jsx
// This module should include only general purpose functions
// see E:\_DDDprojects\_JSX-library

KLD_B.BackupSettings = function () { // ===========================================================
/*            Backup the settings file - it may be modified in this session
Returns       Function value is xxx
Called by     FindRepl_Palette
Calling       Message
Comment       The settings file must be located in the same directory as the script
              The backup file will get the name xxx_backup.xml
Reference     xxx
Used in       FMfindRepl
History       2020-12-14
*/
var bOK, fBackup, fXmlFile, msg_01, sXmlFile, sBackup;

  sXmlFile = KLD_B.APP_NAME + ".xml";             // FMfindRepl.xml
  sBackup  = KLD_B.APP_NAME + "_backup.xml";
  fXmlFile = File($.fileName.replace (/[^\\\/]+$/i , sXmlFile));
  fBackup  = new File($.fileName.replace (/[^\\\/]+$/i , sBackup));

  try {
      fXmlFile.copy(fBackup);
      return  true;
  } catch (e) {                                   // what kind of error can this be ?
      msg_01 = "Copy error on XML file «" + sXmlFile + "». Error:\n "+ e;
      KLD_B.Message ("E", msg_01, "BackupSettings");
      fXmlFile.close();
      return false;
  }
} //--- end BackupSettings ------------------------------------------------------------------------

KLD_B.CopyToClipboard = function (theString) { // ==================================================
/*            Copy string to clipboard for use in other application
Arguments     theString:  string to be copied to clipboard
Called by     FMbiblioCallBibapp
Calling       -
Reference     Based on Rick Quatro's method
*/
var oDoc, tRange, masterPage, textFrame, oPgf, textLoc, PT = 65536;
// Create a textframe with a paragraph on the first masterpage (which always exists)
  tRange = new TextRange;
  oDoc = app.ActiveDoc;
  masterPage = oDoc.FirstMasterPageInDoc;         // Get first master page in document.
  textFrame = oDoc.NewTextFrame (masterPage.PageFrame); // Add a temporary text frame to the master page.
  textFrame.Width  = 480 * PT;
  oPgf = textFrame.FirstPgf;
  textLoc = new TextLoc (oPgf, 0);
  oDoc.AddText (textLoc, theString);              // write string into this paragraph

  tRange.beg.obj = oPgf;                           // select this paragraph
  tRange.beg.offset = 0;
  tRange.end.obj = oPgf;
  tRange.end.offset = theString.length;
  oDoc.TextSelection = tRange;

  oDoc.Copy (0);                                  // copy selection to clipboard not interactive
  textFrame.Delete ();                            // delete the text frame
} // --- end CopyToClipboard ------------------------------------------------------------------------------

KLD_B.FillItemList = function (oList, aSource) { // ==================================================
/*
Arguments     oList:   dialogue list to be filled
              aSource: source array for the fill
Returns       oList filled up
Called by     DebugMenu
Calling       -
*/
var j, nItems;

  nItems = aSource.length;
  oList.removeAll ();                             // clear the list
  for (j = 0; j < nItems; j++) {                  // rebuild list
    oList.add ("item", aSource[j]);
  }
} //--- end FillItemList --------------------------------------------------------------------------

KLD_B.FindAndReplacePara = function (targetDoc, findString, sourceDoc, trReplace, loopMax) { // ==================================================
/*
Arguments     targetDoc:     where to find and replace
              findString:    what to be replaced
              sourceDoc:     whee the replacement TR comes from
              trReplace:     the replacement text range (e.g. a pragraph)
              loopMax:       emergency back door - twice the # of initially found citations
returns       replacemtCounter
Called by     ExpandInDoc
Calling       GetFindParams, InitFA_errno, ReApplyFontAndSize
Comment       Since we have collected only in doc.MainFlowInDoc, we replace only in this flow
              We do not have an initial TextSelection, hence start directly at the first para
              loopMax used as emergency back door - twice the # of initially found citations
Reference     Russ Ward in https://https://forums.adobe.com/thread/895626; 2081392 and Rick Quatro
*/
var tr = new TextRange(),
    restoreTR, frame = 0, loopCounter = 0, replacementCounter = 0,
    findParams = new PropVals(),
    firstPgf = targetDoc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;

  tr.beg.obj = tr.end.obj = firstPgf;             // set up the starting text range as the very beginning
  tr.beg.offset = tr.end.offset = 0;              // of the flow. We'll move straight from beginning to end.
  trSaved = tr                                    // to come back after work
//We don't need the find to wrap because we are controlling the flow from beginning to end.
  findParams = KLD_B.GetFindParams (findString);        // Get the find parameters for finding string, case sensitive

  KLD_B.InitFA_errno ();                                // reset - it is write protected
  tr = targetDoc.Find(tr.beg, findParams);        // and do an initial find to get started.
                                                  // find and replace loop as long as we keep finding
  while(FA_errno === Constants.FE_Success && loopCounter++ < 2*loopMax) {
    targetDoc.TextSelection = tr;                 // set up the text range to clear the original text
    targetDoc.Clear(0);                           // clear it
    app.ActiveDoc = sourceDoc;                    // switch to sourceDoc
    sourceDoc.TextSelection = trReplace;          // to be able to get the replacement text
    sourceDoc.Copy(0);
    app.ActiveDoc = targetDoc;                    // switch to targetDoc
    targetDoc.Paste(0);                           // and paste the replacement text
    KLD_B.ReApplyFontAndSize (targetDoc, tr);             // re-apply the current para format

    if(FA_errno === Constants.FE_Success) {       // increment our return counter - does not work
      replacementCounter++;
    }
    KLD_B.InitFA_errno ();                              // to be used to track the progress of the find and replace
    tr = targetDoc.Find(tr.beg, findParams);      // something screwy happened while we were replacing text.
  }
  targetDoc.ScrollToText(trSaved);                // we're done. Restore the document to it's original area of display
  return replacementCounter;
} // --- end FindAndReplacePara -------------------------------------------------------------------

KLD_B.FindAndReplaceString = function (activeDoc, findString, replaceString, loopMax) { // ==================================================
/*
Arguments     activeDoc:     the name says it
              findString:    what to be replaced
              replaceString: the replacement
              loopMax:       emergency back door - twice the # of initially found citations
Return        Function returns replacemtCounter
Called by     ReplaceInDoc
Calling       GetFindParams, InitFA_errno
Comment       Since we have collected only in doc.MainFlowInDoc, we replace only in this flow
              We do not have an initial TextSelection, hence start directly at the first paragraph
Reference     Russ Ward in https://forums.adobe.com/message/3888653#3888653
*/
var tr = new TextRange(),
    restoreTR, frame = 0, loopCounter = 0, replacementCounter = 0,
    findParams = new PropVals(),
    firstPgf = activeDoc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;

  tr.beg.obj = tr.end.obj = firstPgf;             // set up the starting text range as the very beginning
  tr.beg.offset = tr.end.offset = 0;              // of the flow. We'll move straight from beginning to end.
  trSaved = tr                                    // to come back after work
//We don't need the find to wrap because we are controlling the flow from beginning to end.
  findParams = KLD_B.GetFindParams (findString);        // Get the find parameters for finding string, case sensitive

  KLD_B.InitFA_errno ();                                // reset - it is write protected
  tr = activeDoc.Find(tr.beg, findParams);        // and do an initial find to get started.

  while(FA_errno === Constants.FE_Success && loopCounter++ < 2*loopMax) { //find and replace loop as long as we keep finding
    activeDoc.TextSelection = tr;                 // set up the text range to clear the original text
    activeDoc.Clear(0);                           // clear it
    activeDoc.AddText(tr.beg, replaceString);     // insert the new text at the original beginning of the text range
    tr.beg.offset += replaceString.length;        // lets jimmy the text range in memory to place it directly after
                                                  // the string we just inserted, so the find picks back up after that.
    if(FA_errno === Constants.FE_Success) {       // increment our return counter - does not work
      replacementCounter++;
    }
    KLD_B.InitFA_errno ();                              // to be used to track the progress of the find and replace
    tr = activeDoc.Find(tr.beg, findParams);      // something screwy happened while we were replacing text.
  }
  activeDoc.ScrollToText(trSaved);                // we're done. Restore the document to it's original area of display
  return replacementCounter;
} // --- end FindAndReplaceString ------------------------------------------------------------------------------

KLD_B.GetScriptPath = function () { // ============================================================
/*            Script path without final \
Called by     ReportInfo
Calling       -
Returns       path in the function
*/
var scrPath, newPath, winPath, lastBSlash;

// platform independent format e.g. /e/_DDDprojects/FM+EN-escript/escript
  scrPath = $.fileName;                           // script path (including the file-name of the script)
  winPath = new File (scrPath);
  scrPath = winPath.fsName;                       // get Windows format
  lastBSlash = scrPath.lastIndexOf("\\");         // \ needs escaping
  scrPath = scrPath.substring(0, lastBSlash);
  return scrPath;                                 // no final \ !
} // --- end GetScriptPath ------------------------------------------------------------------------

KLD_B.GetDocPath = function () { // ===============================================================
/*            Document path without final \
Called by     FMbiblioCollect, FMbiblioCallBibapp, FMbiblioResolve, FMbiblioExpand, CreateBibDoc, ReportInfo
Calling       -
Returns       path in the function
*/
var docFile, scrPath, lastBSlash;

  if (app.ActiveBook.ObjectValid()) {
    docFile = app.ActiveBook.Name;
  } else {
    docFile = app.ActiveDoc.Name;
  }
  lastBSlash = docFile.lastIndexOf("\\");         // \ needs escaping
  scrPath = docFile.substring(0, lastBSlash);
  return scrPath;                                 // no final \ !
} // --- end GetDocPath ---------------------------------------------------------------------------

KLD_B.GetIndexFromName = function (aArray, sSearch) { //===========================================
/*            Get Index of particular contents
Arguments     aArray          Array of strings to be searched
              sSearch         String to be found.
                              May be part of what to find - only first occurrence in array element is found
Returns       Index of found item - or null
Comment       Character case and blanks are ignored in both source item and search item
              aArray may contain placeholder null items which are ignored.
Called by     MakerIniRead, MakerIniWrite, ...
Used in       FMgraph, ...
History       2019-04-24
*/
var j, jFound, saLen, sItem, sSearch;

  saLen     = aArray.length;
  sSearch   = sSearch.replace (" ", "");
  sSearch   = sSearch.toUpperCase();
  for (j = 0; j < saLen; j++) {
    if (aArray[j] != null) {
      sItem = aArray[j].toUpperCase();
      sItem   = sItem.replace (" ", "");
      jFound = sItem.indexOf(sSearch);
      if (sItem.indexOf(sSearch) == 0) {          // found at the beginning
        return j;
      }
    }
  }
  return null;
} //--- end GetIndexFromName ----------------------------------------------------------------------

KLD_B.GetText = function (textObj, oDoc) { // ==================================================
/*
Arguments     textObj: any type of text object
              oDoc:    document containing teh text object
returns the    text in the function
Called by     GetBiblioFromRTF, CollectInDoc, CollectInTables, CollectInFootnotes, SkipNonReferenceParagraphs, ReadFileRTF, GetTRafterTAB
Calling       -
*/
var j, text = "", textItems;
  if (textObj.constructor.name !== "TextRange") { // Get a list of the strings in the text object or text range
  textItems = textObj.GetText(Constants.FTI_String);
  } else {
    textItems = oDoc.GetTextForRange(textObj, Constants.FTI_String);
  }
  for (var j = 0; j < textItems.len; j += 1) {    // Concatenate the strings
    text += (textItems[j].sdata);
  }
  return text;                                    // Return the text
} // --- end GetText ------------------------------------------------------------------------------

KLD_B.GetXMLdata = function (sXmlFile) { // =======================================================
/*            Read localisation strings from xml file
Argements     sXmlFile        fully qualified path of xml file or just the file name.
              File must exist
              If no filepath is given (no slash or backslash present),
              then the file is assumed in the same directory as the script
Returns       xml sructure
Usage         xSettings = getXMLdata (sXmlFile);
Called by     SwitchUIlanguageX
Attention     this function is called by SwithUIlanguageX, hence localisation has not finished yet.
              Therefore: error messages need to be localised with the ExtendScript method
Reference     E:\FM-ExtendScript\SaveBookOldVersion\SaveBookAsOldVersion.jsx by Rick Quatro
History       2020-11-11
*/
var e, fXmlFile, xData, msg_01, msg_02;

  msg_01 = { en: "XML file «" + sXmlFile + "» not found.",
             de: "XML datei «" + sXmlFile + "» nicht gefunden.",
             fr: "Fichier XML «" + sXmlFile + "» non trouvé."};
  msg_02 = { en: "Read error on XML file «" + sXmlFile + "». Error:\n "+ e,
             de: "Lesefehler auf XML datei «" + sXmlFile + "» . Fehler:\n "+ e,
             fr: "Erreur de lecture sur le fichier XML «" + sXmlFile + "». Erreur:\n " + e};

  if ((sXmlFile == null) || (sXmlFile == undefined)) {
    return false;
  } else if (sXmlFile.indexOf("/") > 0 || sXmlFile.indexOf("\\") > 0) { // fully qualified path
    fXmlFile = File(sXmlFile);
  } else {
    fXmlFile = File($.fileName.replace (/[^\\\/]+$/i , sXmlFile));  // file in script folder
  }
  if (fXmlFile.exists === false) {
      KLD_B.IconMessage ("E", localize (msg_01), "getXMLdata");
      return false;
  }
  fXmlFile.open("r");
  fXmlFile.encoding = "UTF-8";
  try {
      xData = new XML(fXmlFile.read());
      fXmlFile.close();
      return  xData;
  } catch (e) {                                   // what kind of error can this be ?
      KLD_B.IconMessage ("E", localize (msg_02), "getXMLdata");
      fXmlFile.close();
      return false;
  }
} //--- end getXMLdata ----------------------------------------------------------------------------

KLD_B.IconMessage = function (msgType, msgTxt, msgTitle, parm1) { // ==============================
/*            Message dialogue with icon choiches
Arguments     msgType  (1 character): I (info), E (error), Q (question), W (warning)
              msgText  String may conatin \n (newline), symbolic parameters %01 ... %09
                       \r (same as \n) and \t (tab) not working with Default layout behaviour
              msgTitle Title for the window, will be prefixed wit application-name
              parm1 ... parm9:  strings replacing the symbolic parameters %01 … %09 (unused are skipped)
Returns       -
Comment       Icon files must reside in subdirectory Media
              To be exited via X button
Used in       FMgraph
History       2020-11-04 tested
*/
var j, nArguments, icon, sIconPath, win;

  if (typeof msgTxt == "xml" ) {msgTxt = msgTxt.toString();}
// Following two replacements only neccessary for read-in values, however they do not harm direct string parameters
  msgTxt = msgTxt.replace (/\\n/g , "\n");
  msgTxt = msgTxt.replace (/\\t/g , "\t");        // not working with Default layout behaviour
  sIconPath = KLD_B.GetScriptPath ();
  nArguments = arguments.length - 3               // handle the symbolic parameters
  if (nArguments > 9) {
    nArguments = 9;                               // %10 would be read as %1
  }
  for (j = 0; j <= nArguments; j++) {
    msgTxt = msgTxt.replace ("%0" + j, arguments[j+2]);
  }
// ---------------------------------------------- Icon files are in same directory as top function
  switch (msgType.toUpperCase()) {
    case "I": icon =  new File(sIconPath + "\\icon-info.png");
      break;
    case "W": icon =  new File(sIconPath + "\\icon-warning.png");
      break;
    case "E": icon =  new File(sIconPath + "\\icon-error.png")
      break;
    case "Q": icon =  new File(sIconPath + "\\icon-question.png");
      break;
    default:  icon =  new File(sIconPath + "\\icon-info.png");
    }
// --- define the window with its contents -------
  win = new Window ("dialog", msgTitle) ;
 // --- position the window near the middle of the screen
//var xLoc = $.screens[0].right / 3 * 2;
//var yLoc = $.screens[0].bottom / 3;
//win.preferredSize = [350,100];
//win.location = [xLoc, yLoc];
//win.orientation = "column";
  win.g1 = win.add ('group {orientation: "row"}');
  win.g1.img = win.g1.add("image", undefined, icon);
  win.g1.info = win.g1.add("staticText", undefined, msgTxt, {multiline: true, scrolling: true});
  win.g1.info.graphics.font = ScriptUI.newFont ("", "", 14);
  win.g1.info.minimumSize = [300, 100];
	win.show();
} // --- end IconMessage --------------------------------------------------------------------------

KLD_B.InitFA_errno = function () { // =============================================================
/*
Called by     FindAndReplaceString, FindAndReplacePara
Calling       -
Reference     https://forums.adobe.com/thread/962910
*/
  app.GetNamedMenu("!MakerMainMenu");             //If this fails, you've got bigger problems
  return;
} // --- end InitFA_errno -------------------------------------------------------------------------

KLD_B.IsBookRequiredQ = function (){ // ==================================================
/*
Called by     FMbiblioResolve, FMbiblioExpand
Calling       GetIniValue, StringToBool
*/
var bookIsRequired = false;

  KLD_B.bDoingBook = KLD_B.StringToBool (KLD_B.MakerIniRead ("D+DD", "FMbib_DoingBook")[0]);
  if (KLD_B.bDoingBook) {
    if (app.ActiveBook.ObjectValid()) {
      bookIsRequired =  false;                    // we are in a book
    } else {
      bookIsRequired =  true;                     // we are in a doc
    }
  }
  return bookIsRequired;
} // --- end BookIsRequired ------------------------------------------------------------------------------

KLD_B.MakerIniRead = function (sSection, sNames1) { // ============================================
/*            Read ini item(s) from maker.ini file
              maker.ini of the current FM session
Arguments     sSection        Section in maker.ini
              sNames1…        Value name(s) as used in the ini-file
Returns       Array of read values; Not found items get value null
Called by     SetUpMenus
Calling       Message, GetIndexFromName
Attention     Returning only one value is also an Array item!
              If an item is found prior to the section line, it may be the wrong one.
              It is important to have unique item names in the files independently of sections
              The object property app.UserSettingsDir came to life in FM-12, is not available before
Usage         aMsgSettings   = KLD_G.MakerIniRead ("D+DD", "msg_Info", "msg_Warn", "msg_Error");
              KLD_G.msgInfo  = KLD_G.StringToBool (aMsgSettings[0].toString()); // Informative messages
              KLD_G.msgWarn  = KLD_G.StringToBool (aMsgSettings[1].toString()); // Warnings
              KLD_G.msgError = KLD_G.StringToBool (aMsgSettings[2].toString()); // Error messages
*/
var appdata, jArg, nArguments, sIniFile, jItem, aLines=[], aMatch, bNotFound, oRegEx={},
    jSection, fSettings, sValueName, aValues=[];

  appdata    = app.UserSettingsDir;               // %appdata%\Adobe\FrameMaker\<version>
  sIniFile    = appdata + "\\maker.ini";
  bNotFound = true;
  oRegEx = {
    section: /^\s*\[\s*([^\]]*)\s*\]\s*$/,
    param: /^\s*([\w\.\-\_]+)\s*=\s*(.*?)\s*$/,
    comment: /^\s*;.*$/
  } ;
  fSettings = new File(sIniFile);
  fSettings.open("r");
  if (fSettings.error !== "") {
    KLD_B.IconMessage ("E", KLD_B.UItxt.MakerIniRead.@msg_01, "MakerIniRead", fSettings);
    return "Ini-read-error";
  }
  nArguments = arguments.length;

  while(!fSettings.eof) { // -------------------- read entire file into array
    aLines.push(fSettings.readln());
  }
  fSettings.close();

  bNotFound = true;
  nArguments = arguments.length;
  jSection = KLD_B.GetIndexFromName (aLines, "["+sSection+"]"); // Does section exist
  if (jSection == null) {
    KLD_B.IconMessage ("E", KLD_B.UItxt.MakerIniRead.@msg_02, "MakerIniRead", sSection, fSettings);
    return null;
  } else {                                        // we can look for the items
    for (jArg = 1; jArg < nArguments; jArg++) {   // 1, 2, 3 for n=4
      jItem = KLD_B.GetIndexFromName (aLines, arguments[jArg]);
      if (jItem != null) {
        aMatch = aLines[jItem].match(oRegEx.param);
        sValueName = aMatch[1];
        aValues.push(aMatch[2]);
      } else {
        KLD_B.IconMessage ("W", KLD_B.UItxt.MakerIniRead.@msg_03, "MakerIniRead", arguments[jArg], fSettings);
        aValues.push(0);
      }
    }
  }
  return aValues;
} // -- end MakerIniRead   ------------------------------------------------------------------------

KLD_B.MakerIniWrite = function (sSection, sKey1) { // =============================================
/*            Write ini item(s) to maker.ini file
              maker.ini of the current FM session
Arguments     sSection        Section in maker.ini
              sKey1…          ini entreis in the usual Name = Value pair notation
Returns       true for success, false if any of the writes failed
Called by     MsgHideBtn
Calling       Message, GetIndexFromName
Attention     In newer FM versions (> v14) the list of recently visited files and favourites
              may interfere with this function!
Comment       If section does not exist already, it will be created at the end of ini file
              The object property app.UserSettingsDir came to life in FM-12, is not available before
*/
var appdata, j, jArg, jItem, nArguments, sCurrentLine, sIniFile, aLines=[], nLines, bNotFound, aMatch,
    nOldLines, jSection, oRegEx={}, fSettings, sValueName, sValue;

  oRegEx = {
    section: /^\s*\[\s*([^\]]*)\s*\]\s*$/,
    param: /^\s*([\w\.\-\_]+)\s*=\s*(.*?)\s*$/,
    comment: /^\s*;.*$/
  } ;
  appdata    = app.UserSettingsDir;               // %appdata%\Adobe\FrameMaker\<version>
  sIniFile    = appdata + "\\maker.ini";
  fSettings = new File(sIniFile);
  fSettings.open("r");
  if (fSettings.error !== "") {
    KLD_B.IconMessage ("E", KLD_B.UItxt.MakerIniWrite.@msg_01, "MakerIniWrite", fSettings);
    return false;
  }
  while(!fSettings.eof) { // -------------------- read entire file into array
    aLines.push(fSettings.readln());
  }
  fSettings.close();

  nLines = aLines.length;
  nOldLines = nLines;
  bNotFound = true;
  nArguments = arguments.length;
  for (j = 0; j < nLines; j++ ) {  // ----------- Does section already exist
    aMatch = aLines[j].match(oRegEx.section);
    if (aMatch == null) {continue;}
    if (aMatch[1].toUpperCase() == sSection.toUpperCase()) {
      jSection = j;
      bNotFound = false;
      break;
    }
  }
  if (bNotFound) {  // -------------------------- section does not exsit
    aLines.push("[" + sSection + "]");            // Add section to array
    for (jArg = 1; jArg < nArguments; jArg++) {
      aLines.push(arguments[jArg]);               // Add item to array
    }
  } else { // ----------------------------------- Section exists
    for (jArg = 1; jArg < nArguments; jArg++) {   // 1, 2, 3 for n=4
      aMatch = arguments[jArg].match(oRegEx.param);
      sValueName = aMatch[1];
      sValue     = aMatch[2];
      jItem = KLD_B.GetIndexFromName (aLines, sValueName);
      if (jItem != null) {
        aLines[jItem] = sValueName + " = " + sValue;// replace the ini line
      } else {
        aLines.splice (jSection+1, 0, arguments[jArg]); // Insert after section
      }
    }
  }

  fSettings.open("w"); // ----------------------- Write modified array
  if (fSettings.error !== "") {
    KLD_B.IconMessage ("E", KLD_B.UItxt.MakerIniWrite.@msg_02, "MakerIniWrite", fSettings);
    return false;
  }
  nLines = aLines.length;
  for (j = 0; j < nLines; j++ ) {
    sCurrentLine = aLines[j];               // debug only
    fSettings.writeln(aLines[j]);
  }
  fSettings.close();
  return true;
} // -- end MakerIniWrite -------------------------------------------------------------------------

KLD_B.IconMessage = function (msgType, msgTxt, msgTitle, parm1) { // ==============================
/*            Message dialogue with icon choiches
Arguments     msgType  (1 character): I (info), E (error), Q (question), W (warning)
              msgText  String may conatin \n (newline), symbolic parameters %01 ... %09
                       \r (same as \n) and \t (tab) not working with Default layout behaviour
              msgTitle Title for the window, will be prefixed wit application-name
              parm1 ... parm9:  strings replacing the symbolic parameters %01 … %09 (unused are skipped)
Returns       -
Comment       Icon files must reside in subdirectory Media
              To be exited via X button
Used in       FMgraph
History       2020-11-04 tested
*/
var j, nArguments, icon, sIconPath, win;

  if (typeof msgTxt == "xml" ) {msgTxt = msgTxt.toString();}
// Following two replacements only neccessary for read-in values, however they do not harm direct string parameters
  msgTxt = msgTxt.replace (/\\n/g , "\n");
  msgTxt = msgTxt.replace (/\\t/g , "\t");        // not working with Default layout behaviour
  sIconPath = KLD_B.GetScriptPath ();
  nArguments = arguments.length - 3               // handle the symbolic parameters
  if (nArguments > 9) {
    nArguments = 9;                               // %10 would be read gbFMbib_IsDebugas %1
  }
  for (j = 0; j <= nArguments; j++) {
    msgTxt = msgTxt.replace ("%0" + j, arguments[j+2]);
  }
// ---------------------------------------------- Icon files are in same directory as top function
  switch (msgType.toUpperCase()) {
    case "I": icon =  new File(sIconPath + "\\icon-info.png");
      break;
    case "W": icon =  new File(sIconPath + "\\icon-warning.png");
      break;
    case "E": icon =  new File(sIconPath + "\\icon-error.png")
      break;
    case "Q": icon =  new File(sIconPath + "\\icon-question.png");
      break;
    default:  icon =  new File(sIconPath + "\\icon-info.png");
    }
// --- define the window with its contents -------
  win = new Window ("dialog", msgTitle) ;
 // --- position the window near the middle of the screen
//var xLoc = $.screens[0].right / 3 * 2;
//var yLoc = $.screens[0].bottom / 3;
//win.preferredSize = [350,100];
//win.location = [xLoc, yLoc];
//win.orientation = "column";
  win.g1 = win.add ('group {orientation: "row"}');
  win.g1.img = win.g1.add("image", undefined, icon);
  win.g1.info = win.g1.add("staticText", undefined, msgTxt, {multiline: true, scrolling: true});
  win.g1.info.graphics.font = ScriptUI.newFont ("", "", 14);
  win.g1.info.minimumSize = [300, 100];
	win.show();
} // --- end IconMessage --------------------------------------------------------------------------

KLD_B.ReApplyFontAndSize = function (oDoc, tRange) { // ===================================================
/*
Arguments     oDoc:   where to find the text range
              tRange: arbitrary text range in the current paragraph
Called by     FindAndReplacePara
Calling       -
*/
var pgfFmt = 0, propsPgf, currPgf = oDoc.TextSelection.beg.obj;
  oDoc.TextSelection = tRange;                    // current selection is only part of ¶
  pgfFmt = oDoc.GetNamedPgfFmt(currPgf.Name);     // get properties from catalogue

  if (!pgfFmt.ObjectValid()) {                    // pgf fmt is not in catalogue
    return;
  }
  propsPgf = AllocatePropVals(2);                 // for font-family and -size
  propsCatlg = pgfFmt.GetProps();                 // get properties from catalogue
  j =  GetPropIndex(propsCatlg, Constants.FP_FontFamily);
  propsPgf[0] = propsCatlg [j];
  j =  GetPropIndex(propsCatlg, Constants.FP_FontSize);
  propsPgf[1] = propsCatlg [j];

  tRange.beg.obj = currPgf;                       // select the whole paragraph
  tRange.beg.offset = 0;
  tRange.end.obj = currPgf;
  tRange.end.offset = Constants.FV_OBJ_END_OFFSET;
  oDoc.TextSelection = tRange;
  oDoc.SetTextProps (tRange, propsPgf);          // apply font family and size
} // --- end ReApplyFontAndSize -------------------------------------------------------------------

KLD_B.RefreshDisplay = function () { // ===================================================================
/*
Called by     FMbiblioResolve, FMbiblioExpand
Comment       The following does nothing at all:
                FcodeList = new Array(FCodes.KBD_REFRESHWIN);
                Fcodes(FcodeList);
              The following 'official functions' are not accepted:
                app.ActiveDoc.ReDisplay(); // app.ActiveDoc.ReDisplay is not a function
                app.ActiveDoc.ReFormat();  // app.ActiveDoc.ReFormat is not a function
              Any action on the (hidden) master page does the trick.
TestEnv       FM-14, ES-toolkit CC (4.0.0.1) ES 4.5.5
Calling       -
*/
  app.ActiveDoc.Redisplay();                      // lower cse inside!
/* this works also - but is overkill
  oDoc = app.ActiveDoc
  oMasterPage = oDoc.FirstMasterPageInDoc;
  oTextFrame = oDoc.NewTextFrame (oMasterPage.PageFrame);
  oTextFrame.Delete ();
*/
} // --- end RefreshDisplay -----------------------------------------------------------------------

KLD_B.SetDisplay = function (bWanted) { // ==================================================
/*
Arguments     bWanted: true => set display ON; false => display OFF
Called by     various
Atention      Switching visibility to often lets this function fail.
              See https://forums.adobe.com/thread/2276988
Reference     Rick Quattro
Used in       FMbiblio, FMcalc, FMvars
*/
  if (app.Displaying != bWanted) {
    app.Displaying = bWanted;
    if (KLD_B.bLogging) {Console ("    Visibility set to "  + bWanted + "? It is: " + app.Displaying);}
  }
} //--- end SetDisplay ------------------------------------------------------------------------------

KLD_B.SortUnique = function (aItems) { // ==================================================
/*
Arguments     aItems: array of items which is to be sorted alphabetically
Called by     CreateBibDoc, ReportFoundCitations
Calling       -
Returns       array "unique" in the function
*/
var sorted = [], unique = [], js, ju;
  sorted = aItems.sort();
// alert ("sorted array = " + sorted);
  if (aItems.length === 0) {                      // input may be empty
    return unique;
  }
  if (aItems.lenght === 1) {
    unique = aItems;
    return unique;
  } else {
    js = 1;
    ju = 1;
    unique [0] = sorted [0];
    while (sorted [js]) {
      if (sorted [js-1] !== sorted [js]) {        // skip duplicates
        unique [ju] = sorted [js];
        ju++;
      }
      js++;
    }
    return unique;
  }
} // --- end SortUnique ------------------------------------------------------------------------------

KLD_B.StringToBool = function (sText) { // ========================================================
/*            Interprete strings as true and false
Arguments     string:  string to be interpreted
Returns       Function gets the result
Called by     IsBookRequiredQ, ...
*/
  if (typeof sText != "string" ) {alert ("StringtoBool - sText «" + sText + "» is type" + typeof sText);}

	switch (sText.toLowerCase()) {
		case "true":  case "yes": case "on":  case "ja":   case "ein": case "oui": case "1": return true;
		case "false": case "no":  case "off": case "nein": case "aus": case "non": case "0": return false;
		case null: return false;
		default: return Boolean(string);
	}
} // --- end StringToBool -------------------------------------------------------------------------

KLD_B.SwitchUIlanguageX = function (sLangFile) { // ===============================================
/*            Read the language relevant xml file
Arguments     sLangFile       Base of the languaage xml file. will be appended
                              _de.xml etc.
                              For test pupose this may be the beginning of a real path name
Returns       set global XML object KLD_B.UItxt
Called by     main
Calling       GetXMLdata
Used in       FMfindRepl
History       2020-11-16 test with VSCode and ESDebugger
*/
var appLang, sLangCode;

  appLang = app.Language;
//appLang = Constants.FV_LANG_FRENCH;             // set explictly for UI test
	switch (appLang) {
		case Constants.FV_LANG_ENGLISH:               // 0x01
		case Constants.FV_LANG_BRITISH: {             // 0x02
			KLD_B.UItxt = KLD_B.GetXMLdata (sLangFile + "_en.xml");
			sLangCode = "en";
			break;
		}
		case Constants.FV_LANG_GERMAN: {              // 0x03
			KLD_B.UItxt = KLD_B.GetXMLdata (sLangFile + "_de.xml");
			sLangCode = "de";
			break;
		}
		case Constants.FV_LANG_FRENCH: {              // 0x05
			KLD_B.UItxt = KLD_B.GetXMLdata (sLangFile + "_fr.xml");
			sLangCode = "fr";
			break;
		}
		default: {
			KLD_B.UItxt = KLD_B.GetXMLdata (sLangFile + "_en.xml");
			sLangCode = "en";
		}
	}
	return sLangCode;
} //--- end SwitchUIlanguageX ---------------------------------------------------------------------

KLD_B.WriteXMLdata = function (xmlData, sXmlFile) { // ============================================
/*            Write XML data to file
Argements     xmlData         the full structure to be written.
              sXmlFile        fully qualified path of XML file or just the file name
                              If no filepath is given (no slash or backslash present),
                              then the file is assumed in the same directory as the script.
                              The file is created anew in UTF-8 with BOM
Reference     https://www.davidebarranca.com/2013/03/presets-management-with-dropdownlist-and-xml-in-extendscript/
              Klaus Göbel in https://community.adobe.com/t5/framemaker/xml-and-unicode-a-quarrelling-couple/td-p/11677704
History       2020-12-14
*/
var BOM = "\uFEFF", fXmlFile;

  if ((sXmlFile == null) || (sXmlFile == undefined)) {
    return false;
  } else if (sXmlFile.indexOf("/") > 0 || sXmlFile.indexOf("\\") > 0) { // fully qualified path
    fXmlFile = new File(sXmlFile);
  } else {
    fXmlFile = new File($.fileName.replace (/[^\\\/]+$/i , sXmlFile));  // file in script folder
  }
  try {
    fXmlFile.open('w');
    fXmlFile.encoding = "UTF-8";
    fXmlFile.write(BOM);
    fXmlFile.write(xmlData);
    fXmlFile.close();
  } catch (e) {
    KLD_B.Message ("E", KLD_B.UItxt.WriteXMLdata.@msg_01, "WriteXMLdata", sXmlFile, e.message);
    return false;
  }
  return true;
} // --- End WriteXMLdata -------------------------------------------------------------------------
