﻿// --- FMbiblio_DocObj.jsx ------ UTF-8 -----------------------------------------------------------
// to be included in FMbiblio.jsx

KLD_B.CollectInBook = function (book) { // ========================================================
/*            Collect citations in book
Arguments     book: book object in which to collect
Called by     FMbiblioCollect
Calling       GetOpenParams, CollectInDoc
*/
var oDoc, docName, bookComp = book.FirstComponentInBook;
  while(bookComp.ObjectValid()) {                 // process only FM docs
    if (bookComp.BookComponentFileType === Constants.FV_BK_FM) {
      book.StatusLine = "Processing " + File(bookComp.Name).displayName;
      docName = bookComp.Name;
// doc = SimpleOpen (docName, false);            // Does not handle open errors automatically
      openParams = KLD_B.GetOpenParams ();
      openReturnParams =  new PropVals();
      oDoc = Open (docName, openParams, openReturnParams);
      KLD_B.CollectInDoc (oDoc);
    }
    bookComp = bookComp.NextBookComponentInDFSOrder;
  }
  book.StatusLine = "";
} // --- end CollectInBook ------------------------------------------------------------------------

KLD_B.CollectInDoc = function (oDoc) { // =========================================================
/*            Collect citations in document flow
Arguments     oDoc: document in which to collect
Called by     FMbiblioCollect, CollectInBook
Calling       Message, GetText, GetTempCitations, CollectInFootnotes, CollectInTables
Comment       Flows in anchored frame are not considered
*/
var flow = oDoc.MainFlowInDoc,
    frm = flow.FirstTextFrameInFlow,
    pgf = frm.FirstPgf,
    j, textItems, citations, nCitations;

  if (oDoc.Name === "") {                         // Current document propably \"Untitled\"; The active doc needs to have been saved.
    KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_12, "CollectInDoc");
    return;
  }

  while(pgf.ObjectValid()) {                      // --- find in ordinary paragraphs
    pgfText = KLD_B.GetText (pgf);
    KLD_B.GetTempCitations (pgfText);
    pgf = pgf.NextPgfInFlow
  }

  textItems = flow.GetText(Constants.FTI_FnAnchor); // --- find in footnotes
  for (j = 0; j < textItems.len; j += 1) {
      fn = oDoc.GetUniqueObject(Constants.FO_Fn, textItems[j].obj.Unique);
      KLD_B.CollectInFootnotes (fn, oDoc);
  }

  textItems = flow.GetText(Constants.FTI_TblAnchor); // --- find in tables and table footnotes
  for (j = 0; j < textItems.len; j += 1) {
    tbl = oDoc.GetUniqueObject(Constants.FO_Tbl, textItems[j].obj.Unique);
    KLD_B.CollectInTables (tbl, oDoc);
  }
alert ("¶+Fn+Tfn; # temp citation = " + KLD_B.asFndCitations.length);
} // --- end CollectInDoc -------------------------------------------------------------------------

KLD_B.CollectInTables = function (tbl, oDoc) { // =================================================
/*            Collect citations in table
Arguments     tbl:  Table in which to collect temp. citations
              oDoc: Document with the table
Called by     CollectInDoc
Calling       GetText, GetTempCitations, CollectInFootnotes
*/
var j, cellText = "", cell, citations, fnItems;
  cell = tbl.FirstRowInTbl.FirstCellInRow;

  while (cell.ObjectValid()) {
    cellText = (KLD_B.GetText (cell));
    KLD_B.GetTempCitations (cellText);
    cell = cell.NextCellInTbl;

    fnItems = cell.GetText(Constants.FTI_FnAnchor); // --- are there table footnotes ?
    for (j = 0; j < fnItems.len; j += 1) {
        fn = oDoc.GetUniqueObject(Constants.FO_Fn, fnItems[j].obj.Unique);
        KLD_B.CollectInFootnotes (fn, oDoc);
    }
  }
} // --- end CollectInTables ----------------------------------------------------------------------

KLD_B.CollectInFootnotes = function (fn, doc) { // ================================================
/*            Collect citations in footnotes
Arguments     fn:   footnote/table footnote in which to collect temp. citations
              oDoc: Document with the footnote/table foot note
Called by     CollectInDoc, CollectInTables
Calling       GetText, GetTempCitations
*/
var fnText = "", fnPara, citations;
  fnPara = fn.FirstPgf;

  while (fnPara.ObjectValid()) {
    fnText = (KLD_B.GetText (fnPara));
    KLD_B.GetTempCitations (fnText);
    fnPara = fnPara.NextPgfInFlow;
  }
} // --- end CollectInFootnotes -------------------------------------------------------------------

KLD_B.GetBiblioFromRTF = function (fileName) { // =================================================
/*            GEt bibliographic data from RTF file
Arguments     fileNme:  RTF file name with the formatted citations
Called by     FMbiblioExpand
Calling       GetText, SkipNonReferenceParagraphs, GetTRafterTAB, Message
Comment       The formatted citation is just the reference nummer (BibApp output-format _FMbiblio-expanding)
              It's OK to have less biblio entries then formattet citations - there may be duplicate refs
*/
var nUnres = 0, nBibs = 0, openParams, openReturnParams, pgf, pgfText,
    TAB = String.fromCharCode(8),               // FM has wrong ASCI for TAB
    parts = [],                                 // strings left/right to the TAB on the line
    tabRange;

  openParams = GetOpenDefaultParams ();            // get the default open parameters
  openReturnParams =  new PropVals();
  KLD_B.oRtfDoc = Open (fileName, openParams, openReturnParams);
  if (FA_errno != 0) {
    KLD_B.IconMessage ("E", KLD_B.UItxt.Function.@msg_26, "GetBiblioFromRTF", fileName, FA_errno);
    return false;
  }
  pgf = KLD_B.oRtfDoc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;  // get first pgf in flow

// --- Read the temp/formatted citations --- "formatted" consist of the reference number only
  while (pgf.ObjectValid()) {
    pgfText = KLD_B.GetText (pgf, KLD_B.oRtfDoc);
    if (pgfText.substring (0,2) === "[[") {       // citation lines start with [[
      parts = pgfText.split(TAB);                 // get the two parts of the line
      parts[0] = parts[0].replace("[[", "{");     // replace [[...]] by {...} for find in user doc
      parts[0] = parts[0].replace("]]", "}");
      if (parts[1].substring (0,1) === "{") {     // this placeholder is unresolved, skip it
        nUnres += 1;
        pgf = pgf.NextPgfInFlow;
        continue;
      }
      KLD_B.asFmtCitsRaw.push (parts[0]);        // store the placeholder citation
      KLD_B.asFmtCitsFmt.push (parts[1]);        // store the formatted citation (jut a number)
                                                  // this may be a list of numbers separated by semicolon
      pgf = pgf.NextPgfInFlow;
    } else { break }                              // first line of bibliography encountered
  }
// --- Read the bibliography paragraphs           Citavi (at least v5) starts with a heading and an empty ¶
  pgf = KLD_B.SkipNonReferenceParagraphs (pgf, KLD_B.oRtfDoc);  // cope with differece EndNote - Citavi

  while (pgf.ObjectValid()) {                     // until end of doc
    tabRange = KLD_B.GetTRafterTAB (pgf, KLD_B.oRtfDoc); // Get the text range starting after the TAB to the end of the paragraph.
    if (tabRange) {
      KLD_B.aoBibliography.push(tabRange);       // store the paragraph-part in the array
      nBibs += 1;
      pgf = pgf.NextPgfInFlow;
    } else { break }                              // We are out of the bibliography range
  }
  if (nUnres > 0) {
    KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_18, "GetBiblioFromRTF", KLD_B.sBibAppName, nUnres);
  }
  KLD_B.iCitUnique     = KLD_B.asFmtCitsRaw.length;         // set the statistic data
  KLD_B.iCitUnResolved = nUnres;
  KLD_B.iBibEntries    = nBibs;
// KLD_B.oRtfDoc.Close (Constants.FF_CLOSE_MODIFIED);  // postpone to FMbiblioExpand to keep the paragraph-IDs intact
  return true;
} // --- end GetBiblioFromRTF ---------------------------------------------------------------------

KLD_B.GetFindParams = function (string) { // ======================================================
/*            Get parameters to find a string
Arguments     string: string to be found - inserted into the find parameters
Called by     GetTRafterTAB, FindAndReplaceString, FindAndReplacePara
Calling       -
Returns       find parameters in the function
Consider      string, case;
Reference     Rick Quatro
*/
var findParams = AllocatePropVals (2);

  findParams[0].propIdent.num = Constants.FS_FindText;
  findParams[0].propVal.valType = Constants.FT_String;
  findParams[0].propVal.sval = string;
  findParams[1].propIdent.num = Constants.FS_FindCustomizationFlags;
  findParams[1].propVal.valType = Constants.FT_Integer;
  findParams[1].propVal.ival = Constants.FF_FIND_CONSIDER_CASE;
  return findParams;
} // --- end GetFindParams ------------------------------------------------------------------------

KLD_B.GetOpenParams = function () { // ============================================================
/*            Get parameters to open document without problems
Called by     ReplaceInBook, ExpandInBook, CollectInBook
Calling       -
returns       doc parameters in the function
*/
var params, j;

// --- set up the required parameters
  params = GetOpenDefaultParams();
  j = GetPropIndex(params, Constants.FS_FileIsOldVersion);     // File is from previous version of the FrameMaker
  params[j].propVal.ival = Constants.FV_DoOK;

  j = GetPropIndex(params, Constants.FS_FileIsStructured);     // File is Structured FrameMaker features
  params[j].propVal.ival = Constants.FV_StripStructureAndOpen;

  j = GetPropIndex(params, Constants.FS_FontNotFoundInCatalog);// Catalog contains fonts that aren’t available
  params[j].propVal.ival = Constants.FV_DoOK;

  j = GetPropIndex(params, Constants.FS_FontNotFoundInDoc);    // Document uses unavailable fo
  params[j].propVal.ival = Constants.FV_DoOK;

  j = GetPropIndex(params, Constants.FS_LanguageNotAvailable); // The file uses an unavailable langu
  params[j].propVal.ival = Constants.FV_DoOK;

  j = GetPropIndex(params, Constants.FS_LockCantBeReset);      // File lock could not be reset
  params[j].propVal.ival = Constants.FV_DoOK;

  j = GetPropIndex(params, Constants.FS_RefFileNotFound);      // Document imports another file that isn’t available.
  params[j].propVal.ival = Constants.FV_AllowAllRefFilesUnFindable;

  j = GetPropIndex(params, Constants.FS_FileIsInUse);          // File is already open
  params[j].propVal.ival = Constants.FV_OpenViewOnly;

  j = GetPropIndex(params,Constants.FS_AlertUserAboutFailure); // Alert user if unexpected condition, such as an
  params[j].propVal.ival=false;                                // unrecognized file type, occurs.

  j = GetPropIndex(params,Constants.FS_MakeVisible );          // Make document/book visible as soon it’s opened.
  params[j].propVal.ival=true;                                 // Invisible documents/books can never be active.

  return params;
} // --- end GetOpenParams ------------------------------------------------------------------------

KLD_B.GetTplParams = function () { // =============================================================
/*            Get parameters to open template file
Called by     OpenTemplate
Calling       -
returns       doc parameters in the function
*/
var params, j;

// --- set up the required parameters
  params = GetOpenDefaultParams();
  j = GetPropIndex(params, Constants.FS_RefFileNotFound);
  params[j].propVal.ival = Constants.FV_AllowAllRefFilesUnFindable;

  j = GetPropIndex(params, Constants.FS_FileIsOldVersion);
  params[j].propVal.ival = Constants.FV_DoOK;

  j = GetPropIndex(params, Constants.FS_FontNotFoundInDoc);
  params[j].propVal.ival = Constants.FV_DoOK;

  j = GetPropIndex(params, Constants.FS_LockCantBeReset);
  params[j].propVal.ival = Constants.FV_DoOK;

  j = GetPropIndex(params, Constants.FS_FileIsInUse);
  params[j].propVal.ival = Constants.FV_OpenViewOnly;

  j = GetPropIndex(params,Constants.FS_AlertUserAboutFailure);
  params[j].propVal.ival=Constants.FV_DoCancel;
/*
  j = GetPropIndex(params,Constants.FS_MakeVisible );
  params[j].propVal.ival=false;
*/
  return (params);
} // --- end GetTplParams -------------------------------------------------------------------------

KLD_B.GetTRafterTAB = function (oPgf, doc) { // ===================================================
/*
Arguments     aSearchArray:  array of key/value pairs
Called by     GetBiblioFromRTF
Calling       GetFindParams, GetText
Reference     Lines 08-09 by Russ Ward  saved my project, because otherwise the following statement
Comment       Starts at the beginning of the file, not the current oPgf
*/
var textLoc, textRange, findParams, searchStart, tabRange;
  textLoc = new TextLoc (oPgf, 0);                 // Make a text location at the beginning of the paragraph.
  findParams = KLD_B.GetFindParams ("\x08");            // Get the find parameters for finding an FM TAB.
  searchStart = new TextRange(new TextLoc(oPgf,0), new TextLoc (oPgf, 0));
  doc.TextSelection = searchStart;
  textRange = doc.Find (textLoc, findParams);     // Search for the string.
  if (textRange.beg.obj.ObjectValid ()) {         // See if the tab was found.
  if (textRange.beg.obj.Unique === oPgf.Unique) {
// Select the text after the tab up to the end of the paragraph.
      tabRange = new TextRange (new TextLoc (oPgf, textRange.beg.offset+1), new TextLoc (oPgf, Constants.FV_OBJ_END_OFFSET - 1));
      if ( KLD_B.GetText (tabRange, doc) == "") {
        return 0;                                 // end of flow with empty ¶ encountered
      }
      return tabRange;                            // Return the text range.
    }
  }
  return 0;                                       // TAB Not found
} // --- end GetTRafterTAB ------------------------------------------------------------------------

KLD_B.GetTempCitations = function (pgfText) { // ==================================================
/*
Arguments     pgfText: Paragraph with the temp. citation
Called by     CollectInDoc, CollectInTables, CollectInFootnotes
Calling       Trace
*/
var citations = [], result,                       // array for found citations
    regex = /\{([^\}]+)\}/g;

  while (result = regex.exec (pgfText)) {         // Execute the regular expression
    KLD_B.asFndCitations.push (result[1]);        // Push content of citation (without braces/brackets)
  }
  return;                                         // Return the array
} // --- end GetTempCitations ---------------------------------------------------------------------

KLD_B.OpenTemplate = function () { // =============================================================
/*
Called by     CreateBibDoc
Calling       GetTplParams
return        the Open file in the function
*/
var tplDir, tplFile, openParams, openReturnParams, oFile;
  tplFile = KLD_B.sScriptDir + "\\" + KLD_B.sTemplate;
// oFile = SimpleOpen (tplFile, true);           // does not handle open errors
  openParams = KLD_B.GetTplParams ();
  openReturnParams =  new PropVals();
  oFile = Open (tplFile, openParams, openReturnParams);
  return oFile;
} // --- end OpenTemplate -------------------------------------------------------------------------

KLD_B.ReadFileRTF = function (fileName) { // ======================================================
/*
Arguments     fileName: file name of RTF file which shall be read in
Called by     FMbiblioResolve
Calling       GetText, SkipNonReferenceParagraphs, Message
*/
var nUnres=0, nBibs = 0, openParams, openReturnParams, newDoc, pgf, pgfText,
    TAB = String.fromCharCode(8),               // FM has wrong code for TAB
    parts = [];                                 // strings left/right to the TAB on the line
// clear the global arrays - may contain old stuff
  KLD_B.asFmtCitsRaw = [], KLD_B.asFmtCitsFmt = [], KLD_B.aoBibliography = [];

  openParams = GetOpenDefaultParams();
  openReturnParams =  new PropVals();
  newDoc = Open (fileName, openParams, openReturnParams);
  if (FA_errno != 0) {
    KLD_B.IconMessage ("E", KLD_B.UItxt.Function.@msg_26, "ReadFileRTF", fileName, FA_errno);
    return false
  }
  pgf = newDoc.MainFlowInDoc.FirstTextFrameInFlow.FirstPgf;  // get first pgf in flow
// --- read the temp/formatted citations
  while(pgf.ObjectValid()) {
    pgfText = KLD_B.GetText (pgf, newDoc);
    if (pgfText.substring (0,2) === "[[") {       // citation lines start with [[
      parts = pgfText.split(TAB);                 // get the two parts of the line
      if (parts[1].substring (0,1) !== "{") {
        parts[0] = parts[0].replace("[[", "{");   // replace [[...]] by {...}
        parts[0] = parts[0].replace("]]", "}");
        KLD_B.asFmtCitsRaw.push (parts[0]);      // the placeholder citation
        KLD_B.asFmtCitsFmt.push (parts[1]);      // the formatted citation
      } else {                                    // skip unresolveds
        nUnres += 1;
      }
      pgf = pgf.NextPgfInFlow;
    } else { break }
  }
// brute force method to avoid the random missing first item replacement (repeat first item at end)
  KLD_B.asFmtCitsRaw.push (KLD_B.asFmtCitsRaw[0]);
  KLD_B.asFmtCitsFmt.push (KLD_B.asFmtCitsFmt[0]);

// --- read the bibliography --- only the number of entries used in process 3a;
  pgf = KLD_B.SkipNonReferenceParagraphs (pgf, newDoc); // cope with differece EndNote - Citavi
  while(pgf.ObjectValid()) {                      // until end of doc
    nBibs += 1;                                   // count biblio paragraphs
    pgfText = KLD_B.GetText (pgf, newDoc);
//  KLD_B.aoBibliography.push(pgfText);                // not used in processs 3a
    pgf = pgf.NextPgfInFlow;
  }
  if (nUnres > 0) {
    KLD_B.IconMessage ("I", KLD_B.UItxt.Function.@msg_18, "ReadFileRTF", KLD_B.sBibAppName, nUnres);
  }
  KLD_B.iCitUnique     = KLD_B.asFmtCitsRaw.length; // set the statistic data
  KLD_B.iCitUnResolved = nUnres;
  KLD_B.iBibEntries    = nBibs;

  newDoc.Close (Constants.FF_CLOSE_MODIFIED);
  return true
} // --- end ReadFileRTF --------------------------------------------------------------------------

KLD_B.SaveFileRTF = function (oDoc, fileName, path) { // ==========================================
/*
Arguments     oDoc:     curent document
              fileName: name of the RTF file to be saved to
              path:     path for the save
Called by     FMbiblioCollect
Calling       -
*/
var saveName, saveProps, retProps, j = 0;
  saveName = path + "\\" + fileName;              // must end with .rtf
  retProps = new PropVals();
  saveProps = GetSaveDefaultParams();

  j = GetPropIndex (saveProps, Constants.FS_FileType);
  saveProps[j].propVal.ival = Constants.FV_SaveFmtFilter;

  j = GetPropIndex (saveProps, Constants.FS_SaveFileTypeHint);
  saveProps[j].propVal.sval = "0001ADBIRTF ";     // set filetype hint
//                             0001            ???
//                                 ADBI        vendor ID
//                                     RTF     format ID (blank padded to 4)
// See maker.ini [APIClients]:
// ClientName       =ClientType, facet, format_id, vendor_id, display_name,      description,                   path,                      mode, file extension for filter
// Microsoft RTF 1.6=TextImport, RTF,   RTF,       ADBI,      Microsoft RTF 1.6, Rich Text Format (RTF) Import, filters\ImportUtility.dll, all, ^.rtf
// Microsoft RTF1.6=ExportFilter,RTF,   RTF,       ADBI,      Microsoft RTF1.6,  RTF Export,                    filters\ExportUtility.dll, all, ^.rtf

  oDoc.Save (saveName, saveProps, retProps);      // saveName OK, content is from oDoc

  oDoc.Close (Constants.FF_CLOSE_MODIFIED);       // close collect-file without saving
} // --- end SaveFileRTF --------------------------------------------------------------------------

KLD_B.SkipNonReferenceParagraphs = function (oPgf, oDoc) { // =====================================
/*
Arguments     oPgf: paragraph to be tested
              oDoc: document with the paraph
Returns       the current paragraph object
Called by     GetBiblioFromRTF, ReadFileRTF
Calling       GetText
Comment       At least in Citavi 5 the file with formattet references starts with special ¶ on second section
*/
var sCheck, sNum, loop = 0;
  sCheck = KLD_B.GetText (oPgf, oDoc);
  sNum   = Number(sCheck.substring(0,1));
  while (oPgf.ObjectValid() & (isNaN(sNum) | sCheck.length == 0)) { // text or empty ¶
    oPgf = oPgf.NextPgfInFlow;
    sCheck = KLD_B.GetText (oPgf, oDoc);
    sNum   = Number(sCheck.substring(0,1));
  }
  return oPgf;
} //--- end SkipNonReferenceParagraphs ------------------------------------------------------------
